# Git 协作使用规范

先要配置基本信息，避免用默认信息提交代码，如果是独立的开发机器可以用 `--global` 配置，如果是多人使用的开发机器用 `--local` 配置信息到 repo 本地

```shell
git config --global user.name <your_name>
git config --global user.email <your_email>
```

使用 git 必须通过创建分支进行开发，不要在主干分支上直接开发。
创建本地分支	`git checkout -b branch_name`
删除本地分支	`git branch -d branch_name`

不要粗暴的直接进行 `git add .` 添加所有文件；添加暂存文件时需先运行 `git status` 查看文件状态，最好使用 `git diff` 核对一下改动的内容再运行 `git add` 来添加。

辅助文件如 `.idea .swap ` 和编译产生的目标文件以及数据文件都要添加到 `.gitignore` 中，不要添加到 repo 里

每次要 push 的 commit 里完成一整块功能，提交说明中写清楚该变动的目的；对于本地多条 `git commit` 在准备 push 前先 rebase 来合并 commit，例如本地有三条提交需合成一条：

```shell
git rebase -i HEAD~3 # 指定HEAD前的三条提交
# 将合并页面里的 pick 改为 squash 并保存，然后修改提交信息即可
# 或者撤销 commit 再重新提交
git reset HEAD~3
git add .
git commit -m "merge 3 commit"
# 或者如果是从 develop 分出的 feature/xxx 分支，可以这样合并
git checkout -b feature/xxx
# modify and commit several times, then
git rebase -i --autosquash develop
```

commit 说明信息的格式规范：

```
标题行：50个字符以内，描述主要变更内容

# 主体内容：更详细的说明文本，建议72个字符以内。 需要描述的信息包括:

* 为什么这个变更是必须的? 它可能是用来修复一个bug，增加一个feature，提升性能、可靠性、稳定性等等
* 他如何解决这个问题? 具体描述解决问题的步骤
* 是否存在副作用、风险? 

# 如果需要的化可以添加一个链接到issue地址或者其它文档

```

## 分支管理

* master 分支

master 为主分支，也是用于部署生产环境的分支，确保 master 分支稳定性
master 分支一般由 develop 以及 hotfix 分支合并，任何时间都不能直接修改代码

* develop 分支

develop 为开发分支，始终保持最新完成以及 bug 修复后的代码
一般开发的新功能时，feature 分支都是基于 develop 分支下创建的

* feature 分支

开发新功能时，以 develop 为基础创建 feature 分支
分支命名: "feature/" 开头的为特性分支， 命名规则: "feature/user_module"、 "feature/cart_module"

* release 分支

release 为预上线分支，发布提测阶段，会 release 分支代码为基准提测

* hotfix 分支

分支命名: "hotfix/" 开头的为修复分支，它的命名规则与 feature 分支类似
线上出现紧急问题时，需要及时修复，以 master 分支为基线，创建 hotfix 分支，修复完成后，需要合并到 master 分支和 develop 分支

## 一次提交的完整流程

```shell
# 同步最新的 develop 分支到本地（也是默认的分支）
git fetch
git checkout develop
git pull
# 新建一个 feature/* 分支来做更改
git chekout -b feature/xxxx
# 进行多次更改与本地提交
# 准备 push 到 remote 前先合并提交
git rebase -i --autosquash develop
# 本地分支 push 到 remote 分支
git push origin feature/xxxx
# 到页面上检查当前 push，会有提起 merge request 的选项
```

## Reference

[Git使用规范流程](http://www.ruanyifeng.com/blog/2015/08/git-use-process.html) 
[Git 分支管理最佳实践](https://www.ibm.com/developerworks/cn/java/j-lo-git-mange/index.html)
[GitLab Code Review教程](https://ken.io/note/gitlab-code-review-tutorial)
