<pre>
   _____             __                      ____      ____
  / ___/____  ____  / /_  ____  ____        /  _/___  / __/__  ________  ____  ________
  \__ \/ __ \/ __ \/ __ \/ __ \/ __ \______ / // __ \/ /_/ _ \/ ___/ _ \/ __ \/ ___/ _ \
 ___/ / /_/ / /_/ / / / / /_/ / / / /_____// // / / / __/  __/ /  /  __/ / / / /__/  __/
/____/\____/ .___/_/ /_/\____/_/ /_/     /___/_/ /_/_/  \___/_/   \___/_/ /_/\___/\___/
          /_/
</pre>

# Sophon-Inference

Guide to deploying deep-learning inference networks and deep vision primitives on Sophon TPU.
![image](docs/source/pictures/sophon_inference.png)

### AutoDeploy: Splitting and Compile models for offline preparation.
* Use a general algorithm to split models from different frameworks;
* Support mxnet/tensorflow currnetly, caffe/pytorch for future;
* Integrate BMCompilers (bmnetc/t/m/p);

### SAIL: Sophon Artificial Intelligent Library for online deployment.
* It's a wrapper of bmruntime, bmdnn, bmcv, bmdecoder in BMNNSDK;
* Provide both C++ and Python APIs;
* Automatically manage memory of tensors;

### Algo-kit: Algorithm Toolkit for online deployment of python applications.
* Python algorithm toolkit for deep learing inferecne;
* Contains three types of CV algorithms: classification/detection/segmentation, NLP/Speech for future;
* Relies on SAIL(Python API) and AutoDeploy;

## Prerequisites and Compilation

- **BMNNSDK** Required
- **CMake** Required
- **OpenCV** Required for C++ samples
- **Python3, Numpy** Optional for python samples
- **Boost_python(verison >= 1.6.3)** Optional for python samples
- **Sphinx, Doxygen** Optional for documents

```shell
# tools/install.sh to automatically install the 3rd libs.
cd tools
bash install.sh

# compilation, please set configurations in ./CMakeLists.txt before the following commands.
mkdir build
cd build
cmake ..
make
```

## Samples

### Python samples using AlgoKit

ID|Domain|Model|Platform|Split|Mode|CV Lib|Input
-|-|-|-|-|-|-|-
1|img_cls|googlenet|caffe|N|Static|OpenCV|image
2|img_cls|retnet50|caffe|N|Static|OpenCV|image
3|img_cls|mobilenetv1|caffe|N|Static|OpenCV|image/video
4|img_cls|vgg16|caffe|N|Static|OpenCV|image
5|img_cls|mobilenetv1|tensorflow|N|Static|OpenCV|image
6|obj_det|yolov3|caffe|N|Static|OpenCV|image
7|obj_det|mobilenetyolov3|caffe|N|Static|OpenCV|image
8|obj_det|mobilenetssd|caffe|N|Static|OpenCV|image
9|obj_det|fasterrcnn_vgg|caffe|Y|Static|OpenCV|image
10|obj_det|fasterrcnn_resnet50|tensorflow|Y|Static|OpenCV|image
11|face_det|mtcnn|caffe|N|Dynamic|OpenCV|image
12|face_det|ssh|caffe|N|Static|OpenCV|image
13|sem_seg|deeplabv3_mobilenetv2|tensorflow|Y|Static|OpenCV|image

[Usage instructions](samples/python/README.md)

### C++ samples using SAIL

ID|Domain|Model|Platform|Split|Mode|CV Lib|Input
-|-|-|-|-|-|-|-
1|img_cls|mobilenetv1|caffe|N|Static|OpenCV|image
2|img_cls|mobilenetv1|tensorflow|N|Static|OpenCV|image
3|img_cls|resnet50|tensorflow|N|Static|OpenCV|image
4|obj_det|yolov3|caffe|N|Static|OpenCV|image
5|obj_det|mobilenetssd|caffe|N|Static|OpenCV|image
6|obj_det|mobilenetssd|caffe|N|Static|OpenCV|video
7|obj_det|mobilenetssd|caffe|N|Static|BmCV|video
8|face_det|mtcnn|caffe|N|Dynamic|OpenCV|image

[Usage instructions](samples/cpp_sail/README.md)

### C++ samples using BMNNSDK

ID|Domain|Model|Platform|Split|Mode|CV Lib|Input
-|-|-|-|-|-|-|-
1|img_cls|inceptionv1|caffe|N|Static|OpenCV|image
2|face_det|squeezenet|caffe|N|Static|BmCV|video
3|VPU decode|-|-|-|-|-|video

[Usage instructions](samples/cpp_bmnnsdk/README.md)

## Documnets

* **Sources of documnets:**
1) include/core/*.hpp and include/gift/*.hpp for C++ API.
2) docs/sources/pyhton/*.md for Python API.

* **Generate HTML:**
1) Set WITH_DOC ON in CMakeLists.txt and build.
2) See C++ API documents by opening ./build/docs/cxx_api_doc/html/namespaces.html in a browser.
3) See Python API documents by opening ./build/docs/python_api_doc/index.html in a browser.

## Testing
* **Inference Performance Comparison**

model|batch_size|platform|inference time on SC3(ms)|inference time on P4 without TensorRT(ms)
-|-|-|-|-
mobilessd|1|caffe|3|34.64
mobilenetv1|1|caffe|3|24.78
resnet50|1|caffe|7.875|15.98
vgg16|1|caffe|34|10.46
googlenet|1|caffe|3.5|8.33
mobilenetyolov3|1|tensorflow||67.38
yolov3|1|caffe|47|27.74
mtcnn|1|caffe|84|

* **VPU&BMCV Performance of mobilessd**

Hardware|preprocess(ms)|inference on TPU(ms)|postprocess on CPU(ms)
-|-|-|-
CPU + TPU|1.13(CPU)|3.21|0.65
VPU + TPU|1.72(VPU+TPU)|3.40|0.47

## Deployment

TODO

## Contributing

Please read CONTRIBUTING.md for details on our code of conduct, and the process for submitting pull requests to us.

## Authors

* Haipeng Wan   - Initial work
* Hong Liu      - Initial work
* Juntao Tong   - Initial work
* Lian He       - Initial work
* Tong Liu      - Initial work
* Zhenpeng Xiao - Initial work

See also the list of contributors who participated in this project.

## License

This project is licensed under the Apache License, Version 2.0 - see the LICENSE.md file for details

## Acknowledgments

TODO
