## Image Process Python API

### Frame

an image frame of a video, it's used to receive data read from `VideoDecoder`.

Constructor: `sail.Frame()`

**Args**:

None.

**Returns**:

An instance of Frame.

### ImgFormat

an enumerator denotes the data type of input/output image data.

1. INT: input and output are both int.
2. FLOAT: input and output are both float.
3. INT2FLOAT: input is int while output is flaot.
4. FLOAT2INT: input is float while output is int.

### ImageDecoder

read and decode images.

#### Constructor: `sail.VideoDecoder(input_path)`

**Args**:

input_path: the path of the input image.

**Retruns**:

An instance of ImageDecoder.

#### `sail.ImageDecoder.read(frame)`

Read a image frame from a image.

**Args**:

frame: a Frame instance to receive image data.

**Returns**:

A int. 0 for success, negatives for fail and positives for finish.

### VideoDecoder

read and decode videos.

#### Constructor: `sail.VideoDecoder(input_path)`

**Args**:

input_path: the path of the input video.

**Retruns**:

An instance of VideoDecoder.

#### `sail.VideoDecoder.read(frame)`

Read a image frame from a video.

**Args**:

frame: a Frame instance to receive image data.

**Returns**:

A int. 0 for success, negatives for fail and positives for finish.

#### `sail.VideoDecoder.get_frame_shape()`

Get the shape of frame in the video.

**Args**:

None.

**Returns**:

A Shape instance. [1, 3, height, width]

### BMImgProcess

process images by BM1682.

#### Constructor:

`sail.BMImgProcess()`

**Args**:

None.

**Returns**:

A BMImgProcess instance with a Handler created by itself.

`sail.BMImgProcess(handle)`

**Args**:

handle: a Handler instance.

**Returns**:

A BMImgProcess instance with an existent Handler instance.

#### `sail.BMImgProcess.get_handle()`

**Args**:

None.

**Returns**:

The Handler instance it holds.

#### `sail.BMImgProcess.get_bgr_from_frame(frame, dev_mem)`

Get the BGR format data from a Frame instance read by `VideoDecoder`.

**Args**:

* frame: A Frame instance.
* dev_mem: A Tensor instance.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.resize(input, output, input_shape, output_shape, format)`

Resize the image.

**Args**:

* input: A Tensor instance holds the input data.
* output: A Tensor instance holds the output data.
* input_shape: A Shape instance stands for the input shape.
* output_shape: A Shape instance stands for the output shape.
* format: An ImgFormat instance stands for the data type of input/output.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.crop(input, output, input_shape, top, left, crop_height, crop_width, output_stride, foramt)`

Crop the image.

**Args**:

* input: A Tensor instance holds the input data.
* output: A Tensor instance holds the output data.
* input_shape: A Shape instance stands for the input shape.
* top: int, the vertical coordinate of the top left piont of the crop rectangle.
* left: int, the horizontal ordinate of the top left piont of the crop rectangle.
* crop_height: int, the height of the crop rectangle.
* crop_width: int, the width of the crop rectangle.
* output_stride: int, the stride of output.
* format: An ImgFormat instance stands for the data type of input/output.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.normalize(input, output, shape, scale_b, bias_b, scale_g, bias_g, scale_r, bias_r)`

Normalize the image by multiplying a scale factor and adding a bias in each channel of BGR.

**Args**:

* input: A Tensor instance holds the input data.
* output: A Tensor instance holds the output data.
* shape: A Shape instance stands for the input and output shape.
* scale_b: float, the scale factor in the channel of Blue.
* bias_b: float, the bias in the channel of Blue.
* scale_g: float, the scale factor in the channel of Green.
* bias_g: float, the bias in the channel of Green.
* scale_r: float, the scale factor in the channel of Red.
* bias_r: float, the bias in the channel of Red.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.yuv2bgr(input_y, input_uv, output, shape, y_stride, uv_stride, format)`

Color conversion from yuv to bgr.

**Args**:

* input_y: A Tensor instance holds the input data of color Y.
* input_uv: A Tensor instance holds the input data of color UV.
* output: A Tensor instance holds the output data of color BGR.
* shape: A Shape instance stands for the output shape.
* y_stride:
* uv_stride:
* format: An ImgFormat instance stands for the data type of input/output.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.bgr2rgb(input, output, shape)`

Change the channel order from BGR to RGB.

**Args**:

* input: A Tensor instance holds the input data.
* output: A Tensor instance holds the output data.
* shape: A Shape instance stands for the input and output shape.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.crop_resize_norm(input, output, input_shape, output_shape, format, top, left, crop_height, crop_width, scale_b, bias_b, scale_g, bias_g, scale_r, bias_r)`

Crop, resize and normalize the image.

**Args**:

* input: A Tensor instance holds the input data.
* output: A Tensor instance holds the output data.
* input_shape: A Shape instance stands for the input shape.
* output_shape: A Shape instance stands for the output shape.
* format: An ImgFormat instance stands for the data type of input/output.
* top: int, the vertical coordinate of the top left piont of the crop rectangle.
* left: int, the horizontal ordinate of the top left piont of the crop rectangle.
* crop_height: int, the height of the crop rectangle.
* crop_width: int, the width of the crop rectangle.
* scale_b: float, the scale factor in the channel of Blue.
* bias_b: float, the bias in the channel of Blue.
* scale_g: float, the scale factor in the channel of Green.
* bias_g: float, the bias in the channel of Green.
* scale_r: float, the scale factor in the channel of Red.
* bias_r: float, the bias in the channel of Red.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.resize_norm(input, output, input_shape, output_shape, format, scale_b, bias_b, scale_g, bias_g, scale_r, bias_r)`

Resize and normalize the image.

**Args**:

* input: A Tensor instance holds the input data.
* output: A Tensor instance holds the output data.
* input_shape: A Shape instance stands for the input shape.
* output_shape: A Shape instance stands for the output shape.
* format: An ImgFormat instance stands for the data type of input/output.
* scale_b: float, the scale factor in the channel of Blue.
* bias_b: float, the bias in the channel of Blue.
* scale_g: float, the scale factor in the channel of Green.
* bias_g: float, the bias in the channel of Green.
* scale_r: float, the scale factor in the channel of Red.
* bias_r: float, the bias in the channel of Red.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.crop_norm(input, output, input_shape, format, top, left, crop_height, crop_width, scale_b, bias_b, scale_g, bias_g, scale_r, bias_r)`

Crop and normalize the image.

**Args**:

* input: A Tensor instance holds the input data.
* output: A Tensor instance holds the output data.
* input_shape: A Shape instance stands for the input shape.
* format: An ImgFormat instance stands for the data type of input/output.
* top: int, the vertical coordinate of the top left piont of the crop rectangle.
* left: int, the horizontal ordinate of the top left piont of the crop rectangle.
* crop_height: int, the height of the crop rectangle.
* crop_width: int, the width of the crop rectangle.
* scale_b: float, the scale factor in the channel of Blue.
* bias_b: float, the bias in the channel of Blue.
* scale_g: float, the scale factor in the channel of Green.
* bias_g: float, the bias in the channel of Green.
* scale_r: float, the scale factor in the channel of Red.
* bias_r: float, the bias in the channel of Red.

**Returns**:

An int. 0 for success.

#### `sail.BMImgProcess.calib_camera(dmem, mapx, mapy, height, width)`

**Args**:

* dmem:
* mapx:
* mapy:
* height:
* width:

**Returns**:

An int. 0 for success.
