## NN inference Python API

### `sail.IOMode`

1. SYSI: Input tensors are in system memory while output tensors are in device memory.
2. SYSO: Input tensors are in device memory while output tensors are in system memory.
3. SYSIO: Both input and output tensors are in system memory.
4. DEVIO: Both input and output tensors are in device memory.

### `sail.AsyncStatus`

1. BEGIN: Only copy input tensors of this batch from system to device.
2. PROCESS: Copy input tensors of this batch from system to device, and copy output tensors of last batch from device to system.
3. END: Only copy output tensors of last batch from device to system.

### `sail.Inference`

do inference of a model.

#### Constructor: `sail.Inference(context_dir, chip_name, tpus, sail.IOMode.SYSO, is_dynamic, is_async)`

**Args**:

* context_dir: str, the directory path of the model IRs.
* chip_name: str, name of the hardware. Option: "BM1682_PCIE", "BM1682_SOC", "BM1684_PCIE", "BM1684_SOC".
* tpus: str, IDs of tpu to use, split by comma. e.g "1,2,3", "all" for all available tpus.
* io_mode: sail.IOMode, specify the input/output tensors are in system memory or device memory.
* is_dynamic: bool, True for dynamic models and False for static ones.
* is_async: bool, True for asynchronous mode and False for synchronous.

**Returns**:

A `sail.Inference` instance.

#### `get_bmrt()`

**Args**:

None.

**Returns**:

A `sail.BmRuntime` instance.

#### `get_all_graph_names()`

**Args**:

None.

**Returns**:

A list contains all the graph names loaded from the context_dir.

#### `get_input_tensor_names(graph_name)`

Get all input tensor names of the specified graph.

**Args**:

graph_name: str, the pecified graph name.

**Returns**:

A list contains all the input tensor names of the graph.

#### `get_output_tensor_names(graph_name)`

Get all output tensor names of the specified graph.

**Args**:

graph_name: str, the specified graph name.

**Returns**:

A list contains all the output tensor names of the graph.

#### `get_max_input_shapes(graph_name)`

Get the max shapes of input tensors of the specified graph.

**Args**:

* graph_name: str, a graph name.

**Returns**:

A dict contains all input tensor names and maximum shapes.

#### `set_input_shape(graph_name, shapes)`

Set input tensor shapes when running dynamic models.

**Args**:

* graph_name: str, a graph name.
* shapes: dict, A dict contains all input tensor names and shapes, and shape is np.ndarray.

**Returns**:

None.

#### `get_input_shape(graph_name, tensor_name)`

Get the input shape of the specified graph and tensor.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.

**Returns**:

A np.ndarry of shape.

#### `get_output_shape(graph_name, tensor_name)`

Get the output shape of the specified graph and tensor.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.

**Returns**:

A np.ndarry of shape.

#### `set_input_tensor_sys(graph_name, tensor_name, data)`

Set the system data of input tensor of the specified graph and tensor.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.
* data: numpy.ndarray, the data to be set.

**Returns**:

None

#### `get_input_tensor_sys(graph_name, tensor_name)`

Get the system data of input tensor of the specified graph and tensor.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.

**Returns**:

A np.ndarray contains data of the tensor in system memory.

#### `get_input_tensor_dev(graph_name, tensor_name)`

Get the device data of input tensor of the specified graph and tensor, when the inference mode is sail.TensorSync.SYNC.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.

**Returns**:

A `Tensor` instance holds the system memory of the tensor.

#### `get_input_tensor_dev(graph_name, tensor_name, mode)`

Get the device data of input tensor of the specified graph and tensor, when the inference mode is sail.TensorSync.ASYNC.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.
* mode: sail.TensorMode.

**Returns**:

A `Tensor` instance holds the system memory of the tensor.

#### `get_output_tensor_sys(graph_name, tensor_name)`

Get the system data of output tensor of the specified graph and tensor.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.

**Returns**:

A np.ndarray contains data of the tensor in system memory.

#### `get_output_tensor_dev(graph_name, tensor_name, mode)`

Get the device data of output tensor of the specified graph and tensor, when the inference mode is sail.TensorSync.SYNC.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.

**Returns**:

A `Tensor` instance holds the system memory of the tensor.

#### `get_output_tensor_dev(graph_name, tensor_name, mode)`

Get the device data of output tensor of the specified graph and tensor, when the inference mode is sail.TensorSync.ASYNC.

**Args**:

* graph_name: str, a graph name.
* tensor_name: str, a tensor name of the graph.
* mode: sail.TensorMode.

**Returns**:

A `Tensor` instance holds the system memory of the tensor.

#### `process(graph_name, mode, status)`

Do inference of the specified graph in asynchronous mode.

**Args**:

* graph_name: str, a graph name.
* mode: sail.TensorMode.
* status: sail.AsyncStatus.

**Returns**:

None.

#### `process(graph_name)`

Do inference of the specified graph in synchronous mode.

**Args**:

graph_name: str, a graph name.

**Returns**:

None.
