# **Algorithm Toolkit**

## Algorithm Toolkit Status
> Toolkit Basic/Inference/Algorithm Status

* basic status
  * pysail version: 2.0.2
  * supported mode: BM1682_PCIE (done) | BM1684_PCIE (to do)
  * supported precision: FP32 (done) | INT8 (to do)
  * supported chip_num: one chip (done) | multi chip (to do)

* inference status:

  |      | Static | Dynamic |
  |:----:|:------:|:-------:|
  | Sync |   yes  |   yes   |
  | Async|    -   |    -    |

* algorithm status

  | Models | Caffe | TensorFlow | MXNet | PyTorch |
  |:------:|:-----:|:----------:|:-----:|:-------:|
  | mobilenetssd | √ | - | - | - |
  | yolov3 | √ | - | - | - |
  | mobilenetyolov3 | √ | - | - | - |
  | fasterrcnn | √ | √ | - | - |
  | mtcnn | √ | - | - | - |
  | ssh | √ | - | - | - |
  | googlenet | √ | - | - | - |
  | resnet50 | √ | - | - | - |
  | vgg16 | √ | - | - | - |
  | mobilenetv1 | √ | √ | - | - |


## Toolkit Test
> Supported Python Version: 2/3

* unit test

```bash
# test face detection mtcnn
python test_mtcnn.py
# test face detection ssh
python test_ssh.py
# test object detection yolov3 && mobilenetyolov3 (coco)
python test_yolov3.py
python test_mobilenetyolov3.py
# test object detection ssd (voc)
python test_mobilenetssd.py
# test object detection fasterrcnn (voc)
python test_fasterrcnn
# test classification (ImageNet)
python test_generalclassifier.py && test_resnet50.py && test_mobilenetv1.py && test_mobilenetv1_tf.py
# test tf object detection fasterrcnn_resent50 (coco)
```

* overall test

```bash
# check the algorithms list in algo-kit
python test_algokit --check_list

# test the algorithms in algokit
python test_alogkit.py --algo_type (cls | obj_det | face_det) --algo_name (resnet50 | yolov3 | ssh ...)

```

## Algorithm Toolkit Structure Tree
> Toolkit Overall Structure

```shell
algo-kit/
├── docs
├── models
│   └── bm1682model -> symbol link nntoolchain model_dir
├── README.md
├── requirments.txt
├── algokit
│   ├── algo_cv
│   │   ├── cls
│   │   ├── det
│   │   └── seg
│   ├── algofactory
│   ├── algo_nlp
│   ├── algo_speech
│   ├── engine
│   │   ├── engineconfig
│   │   └── autodeployconfig
│   ├── kitconfig
│   ├── libs
│   │   └── extand_layer
│   └── utils
└── unittests
```
