"""The implementation of object detection fasterrcnn
"""
from __future__ import print_function
from __future__ import division
from typing import Union, List, Dict
from auto_deploy.api import split as splitter
from auto_deploy.api import infer as net
from auto_deploy.api import compile as compiler
import sys, os
import numpy as np
import logging, cv2, yaml, time
from ...engine.base_engine import BaseEngine

class SemanticSegmentationDEEPLABV3_MOBILENETV2_TF(BaseEngine):
    """Construct deeplabv3_mobilenetv2 semantic segmemtation
    """
    def __init__(self, source_path,
                 subgraph_path, tfmodel_path,
                 framework, input_names,
                 output_names, input_shapes,
                 layout, is_dynamic,
                 process_size, target,
                 conf_threshold):
        super(SemanticSegmentationDEEPLABV3_MOBILENETV2_TF, self).__init__()
        # process_size: (h, w)
        # image will be resize before processing detection
        logging.info("deeplabv3_mobilenetv2 semantic segmemtation model initialization")
        self.subgraph_path = subgraph_path
        self.tfmodel_path = tfmodel_path
        self.framework = framework
        self.input_names = input_names
        self.input_shapes = input_shapes
        self.process_size = process_size
        self.conf_threshold = conf_threshold
        self.tensors_dict = {}
        if len(self.input_names) == len(self.input_shapes):
            for input_name, input_shape in zip(input_names, input_shapes):
                self.tensors_dict[input_name] = np.ndarray(input_shape, dtype=np.float32)
        else:
            raise ValueError('input names and input shapes sizes do not match!')
        self.output_names = output_names
        self.layout = layout
        self.is_dynamic = is_dynamic
        self.target = target
        # check the subgraph file
        if not os.path.isdir(self.subgraph_path):
            print("attention please: this model needs to be split...")
            # autodeploy split
            splitter(self.framework, self.tensors_dict, self.subgraph_path, self.tfmodel_path, \
                params_path=None, outputs=self.output_names, dynamic=self.is_dynamic, layout=self.layout)
            print("split done!")
            compiler(self.subgraph_path, optimize=1, compare=True, target=self.target)
            print("compile done!")

    def predict(self, images):
        """deeplabv3_mobilenetv2 forward
        """
        self.time.tic()
        if isinstance(images, list):
            use_batch = True
        else:
            logging.debug("input images will be forced to a list of np.ndarray")
            use_batch = False
            images = [images]

        # inference
        for image in images:
            origin_size = image.shape
            # this version deeplabv3_mobilenetv2 input size fix in (513,513)
            data, rescale_param = self.rescale_image(image, self.process_size, True)
            data = data[..., [2, 1, 0]] # BGR2RGB PIL.Image read as RGB
            input_data = {self.input_names[0]: np.array([data])}
            out = net(self.subgraph_path, input_data)
            print("inference done!")
            semanticpredictions = out['SemanticPredictions:0']
        if use_batch:
            return semanticpredictions
        else:
            return semanticpredictions
