"""autodeploy algo factory
create the algorithm from autodeploy, based on autodeploy
three stages:
    1. split model graph
    2. compile subgraph (bmnetT/bmnetM)
    3. run algorithm with tpu&&cpu
detector/classifier in this factory is different from original caffe
"""
from __future__ import print_function
import os, sys
from ..kitconfig import ObjDetModel, SegmentationModel
from . basefactory import BaseFactory
from ..utils.utils import load_autodeploy_config

# global param for fasterrcnn_resnet50_tf
default_fasterrcnn_resnet50_tf_param = {'detected_size': (600, 800),
                            'conf_threshold': 0.5}
# global param for deeplabv3_mobilenetv2_tf
default_deeplabv3_mobilenetv2_tf_param = {'process_size': (513, 513),
                            'conf_threshold': 0.5}

class ObjectDetector(BaseFactory):
    def __init__(self, model_path=None):
        super(ObjectDetector, self).__init__(model_path)
    
    def create(self, detector_model, mode, config_path=None, extra_args={}):
        if detector_model is ObjDetModel.FASTERRCNN_RESNET50_TF:
            from ..algo_cv.det.object_detection_fasterrcnn_resnet50_tf \
                import ObjectDetectionFASTERRCNN_RESNET50_TF as Detection
        else:
            raise ValueError('not a valid detector_model: ', detector_model)
        detector_config = load_autodeploy_config(self.model_path, \
            mode, detector_model.value, config_path)
        detector_config = load_extra_param(detector_model, \
            detector_config, extra_args)
        detector = Detection(**detector_config)
        return detector

class SemanticSegment(BaseFactory):
    def __init__(self, model_path=None):
        super(SemanticSegment, self).__init__(model_path)

    def get_supported_models(self):
        return list(algo_name.value for algo_name in SegmentationModel)
    
    def create(self, semantic_model, mode, config_path=None, extra_args={}):
        if semantic_model is SegmentationModel.DEEPLABV3_MOBILENETV2_TF:
            from ..algo_cv.seg.semantic_segmentation_deeplabv3_mobilenetv2_tf \
                import SemanticSegmentationDEEPLABV3_MOBILENETV2_TF as Segmentation
        else:
            raise ValueError('not a valid detector_model: ', semantic_model)
        semantic_config = load_autodeploy_config(self.model_path, \
            mode, semantic_model.value, config_path)
        semantic_config = load_extra_param(semantic_model, \
            semantic_config, extra_args)
        segment = Segmentation(**semantic_config)
        return segment

def load_extra_param(model, arch_config, extra_args):
    """unified all arguments to one dict
    """
    if model is ObjDetModel.FASTERRCNN_RESNET50_TF:
        default_param = default_fasterrcnn_resnet50_tf_param
    elif model is SegmentationModel.DEEPLABV3_MOBILENETV2_TF:
        default_param = default_deeplabv3_mobilenetv2_tf_param
    else:
        raise ValueError('not a valid model:', model)
    for key in extra_args:
        if key not in arch_config:
            arch_config[key] = extra_args[key]
    for key in default_param:
        if key in arch_config:
            pass
        else:
            arch_config[key] = default_param[key]
    return arch_config