"""general classification factory
create various general classifier
"""
from __future__ import print_function
from ..utils.utils import load_predefined_config, load_config
from ..kitconfig import ChipMode, ClassificationModel
import os, sys
from . basefactory import BaseFactory

localpath = os.path.dirname(os.path.realpath(__file__))

default_googlenet_param = {'detected_size': (224, 224),
                       'num_classes': 1000}
default_vgg16_param = {'detected_size': (224, 224),
                       'num_classes': 1000}
default_resnet50_param = {'detected_size': (224, 224),
                       'num_classes': 1000}
default_mobilenetv1_param = {'detected_size': (224, 224),
                       'num_classes': 1000}
default_mobilenetv1_tf_param = {'detected_size': (224, 224),
                       'num_classes': 1001}

class GeneralClassifier(BaseFactory):
    def __init__(self, model_path=None):
        super(GeneralClassifier, self).__init__(model_path)

    def get_supported_models(self):
        return list(algo_name.value for algo_name in ClassificationModel)

    def create(self, classifier_model, mode,
            config_path=None, extra_args={}):
        if classifier_model in \
            [ClassificationModel.GOOGLENET, ClassificationModel.VGG16, \
            ClassificationModel.MOBILENETV1, ClassificationModel.RESNET50]:
            from ..algo_cv.cls.general_classification \
                import GeneralClassification as Classification
        elif classifier_model in \
            [ClassificationModel.MOBILENETV1_TF]:
            from ..algo_cv.cls.general_classification_tf \
                import GeneralClassification_TF as Classification
        else:
            raise ValueError('not a valid classifer_model:', classifier_model)
        if config_path is None:
            arch_config = load_predefined_config(\
                self.model_path, mode, classifier_model.value)
        else:
            arch_config = load_config(config_path, self.model_path, mode)
        arch_config = self.load_param(classifier_model, arch_config, extra_args)
        classifier = Classification(**arch_config)
        return classifier

    def set_param(self):
        pass

    def load_param(self, classifier_model, arch_config,
            extra_args):
        # unified all args to one
        if classifier_model is ClassificationModel.GOOGLENET:
            default_param = default_googlenet_param
        elif classifier_model is ClassificationModel.VGG16:
            default_param = default_vgg16_param
        elif classifier_model is ClassificationModel.RESNET50:
            default_param = default_resnet50_param
        elif classifier_model is ClassificationModel.MOBILENETV1:
            default_param = default_mobilenetv1_param
        elif classifier_model is ClassificationModel.MOBILENETV1_TF:
            default_param = default_mobilenetv1_tf_param
        else:
            raise ValueError('not a valid classifier_model:', classifier_model)

        for key in extra_args:
            if key not in arch_config:
                arch_config[key] = extra_args[key]
        for key in default_param:
            if key in arch_config:
                pass
            else:
                arch_config[key] = default_param[key]
        return arch_config
