"""The function of config file load
"""
import json, yaml
import os, collections, sys
from ..kitconfig import ChipMode

def read_config(file_path):
    """ Load the json file
    """
    with open(file_path, 'r') as f:
        json_object = json.load(f)
    return json_object


def dump_json(file_path, data):
    """ Dump the json file
    """
    with open(file_path, 'w') as fp:
        json.dump(data, fp)
    return

def load_yaml(yaml_path):
    """Load the yaml|json file
    
    Returns:
        A dict which contains the config info
    """
    with open(yaml_path, 'r') as f:
        x = yaml.load(f)
    return x

def load_config(config_path, model_dir, mode):
    """Extract config info from input json file
    """
    whole_config = load_yaml(config_path)
    # whole_config = read_config(config_path)
    arch = whole_config.get('arch', None)
    if arch is None:
        # one more net
        for key in whole_config:
            if 'xform' not in key:
                assert ('arch' in whole_config[key])
                whole_config[key]['arch'] = process_arch(whole_config[key]['arch'], model_dir, mode)
                whole_config[key]['mode'] = mode
    else:
        whole_config['arch'] = process_arch(whole_config['arch'], model_dir, mode)
        whole_config['mode'] = mode
    return whole_config

def process_arch(config, model_dir, mode):
    """Parsing configuration file
    """
    if mode in [ChipMode.BM1682, ChipMode.BM1684]:
        # if mode in [ChipMode.BM1682]:
        #     dir_name = 'bm1682model'
        # else:
        #     dir_name = 'bm1684model'
        # model_dir = os.path.join(model_dir, dir_name)
        for key in config:
            if key == 'context_path':
                config[key] = os.path.join(model_dir, config[key])
            if key == 'input_shapes':
                config[key] = [tuple(config[key][i]) for i in range(len(config[key]))]
            # other key: async dynamic tpu
        return config
    else:
        raise ValueError('not valid mode: {mode}'.format(mode))

def load_predefined_config(model_dir, mode, arch):
    """Extract config info from predefined json file
    """
    localpath = os.path.dirname(os.path.realpath(__file__))
    if not isinstance(mode, ChipMode):
        raise ValueError('not valid mode: {}'.format(mode))
    if mode in [ChipMode.BM1682]:
        chip_name = 'bm1682'
    elif mode in [ChipMode.BM1684]:
        chip_name = 'bm1684'
    else:
        raise ValueError('not valid mode: {}'.format(mode))
    config_path = os.path.join(localpath, '../engine/engineconfig/{}_{}.json'.format(chip_name, arch))
    return load_config(config_path, model_dir, mode)

def load_autodeploy_config(model_dir, mode, arch, config_full_path=None):
    """Extract autodeploy config info from predefined json file
    """
    localpath = os.path.dirname(os.path.realpath(__file__))
    if not isinstance(mode, ChipMode):
        raise ValueError('not valid mode: {}'.format(mode))
    if mode in [ChipMode.BM1682]:
        chip_name = 'bm1682'
        # dir_name = 'bm1682model'
        Target = 'BM1682'
    elif mode in [ChipMode.BM1684]:
        chip_name = 'bm1684'
        # dir_name = 'bm1684model'
        Target = 'BM1684'
    else:
        raise ValueError('not valid mode: {}'.format(mode))
    if config_full_path is None:
        config_path = os.path.join(localpath, '../engine/autodeployconfig/{}_{}.json'.format(chip_name, arch))
    else:
        config_path = config_full_path
    config_dict = load_yaml(config_path)
    for key in config_dict:
        if key in ['source_path', 'subgraph_path', 'tfmodel_path']:
            # config_dict[key] = os.path.join(model_dir, dir_name, config_dict[key])
            config_dict[key] = os.path.join(model_dir, config_dict[key])
    config_dict['target'] = Target
    return config_dict