"""The function of visualization
"""
import numpy as np
import cv2
from ..algo_cv.det.detconfig import get_labels

def draw_face_bbox(image, bboxes, probs, points=None):
    """Draw detected face on the original image

    Args:
        image(np.ndarray). input image
        bboxes(list). detected bbox
        probs(list). confidence of bbox
        points(list). face landmark
    """
    draw = image.copy()
    if points is None:
        points = [None for i in range(len(bboxes))]
    for bbox, prob, point in zip(bboxes, probs, points):
        cv2.putText(draw,str(prob)[:8],(int(bbox[0]),int(bbox[1]-2)), \
                cv2.FONT_HERSHEY_SIMPLEX, 0.3,(0,255,0), 1)
        cv2.rectangle(draw,
                      pt1=(int(bbox[0]), int(bbox[1])),
                      pt2=(int(bbox[2]), int(bbox[3])),
                      color=(255, 0, 0),  # (B, G, R)
                      thickness=2)
        if point is not None:
            for idx in range(5):
                cv2.circle(draw,(int(point[idx]),int(point[idx+5])), \
                    2,(0,0,255))
    return draw

def draw_obj_bbox(image, bboxes, classes, probs, type):
    """Draw detected object on the original image
    """
    draw = image.copy()
    label_list = get_labels(type)
    for bbox, cls, prob in zip(bboxes, classes, probs):
        label = "{}:{:.6f}".format(label_list[cls], prob)
        color = tuple(map(int, np.uint8(np.random.uniform(0, 255, 3))))
        cv2.rectangle(draw,
                      pt1=(int(bbox[0]), int(bbox[1])),
                      pt2=(int(bbox[2]), int(bbox[3])),
                      color=color, thickness=1)
        t_size = cv2.getTextSize(label, cv2.FONT_HERSHEY_SIMPLEX, 0.5 , 1)[0]
        pt2 = int(bbox[0]) + t_size[0] + 3, int(bbox[1]) + t_size[1] + 4
        cv2.rectangle(draw, pt1=(int(bbox[0]), int(bbox[1])),
                      pt2=tuple(pt2), color=color, thickness=-1)
        cv2.putText(draw, label, (int(bbox[0]), t_size[1] + 4 + int(bbox[1])),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.5, (255,255,255), 1)
    return draw