import tensorflow as tf
from google.protobuf import text_format


def node_def_add_node(sss):
  """ add node{} to a string of an op.

  Args:
    sss: original string of an op.

  Returns:
    new string with node{} added.
  """
  sss = sss.replace('\n', '\n\t')
  sss = "node {\n\t" + sss
  sss = sss[0:len(sss)-1] + '}\n'
  return sss

def node_def_remove_colocations(s):
  idx = s.find('loc:@')
  if idx < 0:
    return s
  idx1 = s.find('"', idx)
  to_remove = s[idx:idx1]
  s = s.replace(to_remove, '')
  return s
  
def if_this_input_can_remove(tfgraph, iop):
  op = tfgraph.get_operation_by_name(iop)
  if op.type == 'Const':
    return True, [iop]
  if op.type == 'Identity':
    if len(op.inputs) == 1:
      inputop = op.inputs[0].op
      if inputop.type == 'Const':
        return True, [iop, inputop.name]
  return False, None
  
def node_def_remove_control_inputs(s):
  idx = s.find('input: "^')
  while idx > 0:
    idx_e = s.find('"', idx + 10)
    tmpstr = s[idx:(idx_e+1)]
    s = s.replace(tmpstr, '')
    idx = s.find('input: "^')
  return s
  
def node_def_remove_inputs(s):
  idx = s.find('input: "')
  while idx > 0:
    idx_e = s.find('"', idx + 9)
    tmpstr = s[idx:(idx_e+1)]
    s = s.replace(tmpstr, '')
    idx = s.find('input: "')
  return s
  
# only for debug
def just_write_text(sss, pb_path, pb_name):
  tid = sss.find('tensor_content:')
  while tid > 0:
    eid = sss.find('}', tid)
    sss = sss[:tid] + sss[eid:]
    tid = sss.find('tensor_content:')
  with open(os.path.join(pb_path, pb_name+'txt'), 'w+') as fff:
    fff.write(sss)

def save_str_to_pb(sss, pb_path, pb_name):
  #just_write_text(sss, pb_path, pb_name)
  graph_def = tf.GraphDef()
  text_format.Merge(sss, graph_def)
  tf.train.write_graph(graph_def, pb_path, pb_name, as_text = False)
  

def infer_tensorflow_graph(graph_path, input_names, input_values, \
          output_names, warm_loop=0, infer_loop=1, with_flops=False):
  graph = tf.Graph()
  with graph.as_default():
    graph_def = tf.GraphDef()
    with tf.gfile.GFile(graph_path, 'rb') as fid:
      serialized_graph = fid.read()
      graph_def.ParseFromString(serialized_graph)
      tf.import_graph_def(graph_def, name='')
      with tf.Session() as sess:
        for i in range(warm_loop):
          rets = sess.run(output_names, \
                    feed_dict=dict(zip(input_names, input_values)))
        t1 = time.time()
        for i in range(infer_loop):
          rets = sess.run(output_names, \
                    feed_dict=dict(zip(input_names, input_values)))
        t2 = time.time()
        ff = None
        if with_flops:
          flops = tf.profiler.profile(graph, \
                  options=tf.profiler.ProfileOptionBuilder.float_operation())
          ff = flops.total_float_ops
  return rets, (t2-t1)/infer_loop, ff


def tensorflow_infer_with_performance(graph_infos, graph_id, tensor_values):
  graph_path = graph_infos["graphs"][graph_id]["model_info"]["model_path"]
  input_names = graph_infos['graphs'][graph_id]['inputs']
  input_values = []
  for n in input_names:
    input_values.append(tensor_values[n])
  output_names = graph_infos['graphs'][graph_id]['outputs']
  output_values, _t, _f = infer_tensorflow_graph(graph_path, input_names, \
                              input_values, output_names, 10, 100, True)
  outputs = dict(zip(output_names, output_values))
  return outputs, _t, _f


def tensorflow_infer(graph_infos, graph_id, tensor_values):
  graph_path = graph_infos["graphs"][graph_id]["model_info"]["model_path"]
  input_names = graph_infos['graphs'][graph_id]['inputs']
  input_values = []
  for n in input_names:
    input_values.append(tensor_values[n])
  output_names = graph_infos['graphs'][graph_id]['outputs']
  output_values, _t, _f = infer_tensorflow_graph(graph_path, input_names, \
                                                  input_values, output_names)
  outputs = dict(zip(output_names, output_values))
  return outputs

def get_all_ops_from_model_path(model_path):
  assert(model_path.endswith(".pb"))
  graph = tf.Graph()
  with graph.as_default():
    graph_def = tf.GraphDef()
    with tf.gfile.GFile(model_path, "rb") as fid:
      serialized_graph = fid.read()
      graph_def.ParseFromString(serialized_graph)
      tf.import_graph_def(graph_def, name='')
  ops = graph.get_operations()
  return ops, graph

def get_graph(model_path, device):
  assert(model_path.endswith(".pb"))
  graph = tf.Graph()
  with graph.as_default():
    with tf.device(device):
      graph_def = tf.GraphDef()
      with tf.gfile.GFile(model_path, "rb") as fid:
        serialized_graph = fid.read()
        graph_def.ParseFromString(serialized_graph)
        tf.import_graph_def(graph_def, name='')
  return graph


def tensorflow_infer_outputs(model, input_dict, output_names):
  if type(model) == type(str()):
    _, graph = get_all_ops_from_model_path(model)
  else:
    graph = model
  with graph.as_default():
    with tf.Session() as sess:
      ret = sess.run(output_names, input_dict)
  return ret

def get_tensor_shape_in_graph(graph, start_tensors, \
                              start_shapes, out_tensor_name):
  op = graph.get_operation_by_name(out_tensor_name[:out_tensor_name.find(':')])
  if len(op.outputs) < 1:
    return tuple([-1])
  inputs = []
  for s in start_tensors:
    inputs.append(graph.get_tensor_by_name(s))
  values = []
  for s in start_shapes:
    values.append(np.ones(s))
  with graph.as_default():
    with tf.Session() as sess:
      ret = sess.run(out_tensor_name, feed_dict=dict(zip(inputs, values)))
  # it's a tuple
  return ret.shape
  
def get_flops(model_path):
  """ get flops of a model
  
  Args:
    model_path: A string, path of the model.
    
  Returns:
    An integer, flops of this model
  
  """
  ops, graph = get_all_ops_from_model_path(model_path)
  flops = tf.profiler.profile(graph, \
            options=tf.profiler.ProfileOptionBuilder.float_operation())
  return flops.total_float_ops
  