from auto_deploy.common.base_splitter import Splitter
from auto_deploy.external.tensorflow_functions import get_all_ops_from_model_path
from auto_deploy.external.tensorflow_functions import node_def_add_node
from auto_deploy.external.tensorflow_functions import if_this_input_can_remove
from auto_deploy.external.tensorflow_functions import node_def_remove_control_inputs
from auto_deploy.external.tensorflow_functions import node_def_remove_inputs
from auto_deploy.external.tensorflow_functions import node_def_remove_colocations
from auto_deploy.external.tensorflow_functions import save_str_to_pb
from auto_deploy.external.tensorflow_functions import tensorflow_infer_outputs

import os
import bmnett as bm
import numpy as np
import tensorflow as tf


class TensorflowSplitter(Splitter):
  def initialize(self):
    self.platform = 'tensorflow'
    model_path = self.model_descriptor['model_path']
    ops, graph = get_all_ops_from_model_path(model_path)
    self.ops = ops
    self.graph = graph
    self.input_tensors = self.model_descriptor['input_tensors']
    self.input_names = self.input_tensors.keys()
    self.output_names = self.model_descriptor['output_names']
    self.input_ops = [i[:i.find(':')] for i in self.input_names]
    self.output_ops = [i[:i.find(':')] for i in self.output_names]


  def get_op_name(self, op):
    return op.name

  def is_op_support_old(self, op):
    support_list = [
    'Assert',
    'Placeholder',
    'Conv2D',
    'BiasAdd',
    'Relu',
    'Add',
    'Const',
    'AvgPool',
    'ConcatV2',
    'DepthwiseConv2dNative',
    'FusedBatchNorm',
    'Identity',
    'MatMul',
    'MaxPool',
    'Mean',
    'Mul',
    'Pad',
    'Pack',
    'Relu6',
    'Reshape',
    'Rsqrt',
    'Shape',
    'Softmax',
    'Split',
    'SpaceToBatchND',
    'BatchToSpaceND',
    'Squeeze',
    'StridedSlice',
    'Sub',
    'ResizeBilinear',
    'ArgMax-',
    ]
    if op.type in support_list:
      return True
    return False
  
  def if_op_const(self, op):
    if op.type == 'Const':
      return True
    if op.type == 'Identity':
      return self.if_op_const(op.inputs[0].op)
    return False
  
  def is_op_support(self, op):
    unsupports = ['CropAndResize', 'StridedSlice-', 'Softmax-']
    supports = ['Reshape', 'Split']
    if op.type in unsupports:
      return False
    if op.type in supports:
      return True
    op_dict = dict()
    op_dict['type'] = op.type
    op_dict['dynamic'] = self.dynamic
    op_dict['inputs'] = list()
    op_dict_1 = dict()
    op_dict_1['type'] = op.type
    op_dict_1['dynamic'] = self.dynamic
    op_dict_1['inputs'] = list()
    for i in op.inputs:
      #if i.op.type == 'Const':
      if self.if_op_const(i.op):
        op_dict['inputs'].append({
                                  'shape': None,
                                  'dtype': str(i.dtype).split(' ')[1][1:-2].replace('float', 'fp'),
                                  'is_constant': True})
        op_dict_1['inputs'].append({
                                  'shape': None,
                                  'dtype': str(i.dtype).split(' ')[1][1:-2],
                                  'is_constant': True})
      else:
        op_dict['inputs'].append({
                                  'shape': None,
                                  'dtype': str(i.dtype).split(' ')[1][1:-2].replace('float', 'fp'),
                                  'is_constant': False})
        op_dict_1['inputs'].append({
                                  'shape': None,
                                  'dtype': str(i.dtype).split(' ')[1][1:-2],
                                  'is_constant': False})
    is_support = bm.op_is_supported(op_dict)
    is_support_1 = bm.op_is_supported(op_dict_1)
    return is_support or is_support_1
    support_list = [
    'Assert',
    'Placeholder',
    'Conv2D',
    'BiasAdd',
    'Relu',
    'Add-',
    'Const',
    'AvgPool',
    'ConcatV2-',
    'DepthwiseConv2dNative',
    'FusedBatchNorm',
    'Identity',
    'MatMul',
    'MaxPool',
    'Mean',
    'Mul-',
    'Pad',
    'Pack',
    'Relu6',
    'Reshape-',
    'Rsqrt',
    'Shape-',
    'Softmax',
    'Split-',
    'SpaceToBatchND',
    'BatchToSpaceND',
    'Squeeze',
    'StridedSlice',
    'Sub-',
    'ResizeBilinear',
    'ArgMax-',
    ]
    #if not is_support and op.type in support_list:
      #print(op.name)
      #print(op_dict)
      #exit(-1)
    #return bm.op_is_supported(op_dict)
    
  def is_op_support_old2(self, op):
    unsupports = ['CropAndResize']
    if op.type in unsupports:
      return False
    inputattrs = []
    for i in op.inputs:
      if i.op.type == 'Const':
        inputattrs.append(bm.InputAttr(str(i.dtype).split(' ')[1][1:-2], True))
      else:
        inputattrs.append(bm.InputAttr(str(i.dtype).split(' ')[1][1:-2], False))
    return bm.op_is_supported(op.type, inputattrs, self.dynamic)

  def is_op_compute(self, op):
    compute_list = [
    'Conv2D',
    'BiasAdd',
    'Relu',
    'AvgPool',
    'DepthwiseConv2dNative',
    'FusedBatchNorm',
    'MatMul',
    'MaxPool',
    'Pad',
    'Relu6',
    'Softmax',
    ]
    if op.type in compute_list:
      return True
    return False

  def is_op_dangerous(self, op):
    dangerous_list = [
    'NextIteration',
    'LoopCond',
    'Pack',
    'Switch',
    'Shape',
    'StridedSlice',
    'Split',
    'LogicalAnd',
    'Assert--',
    'TensorArrayV3',
    'SpaceToBatchND',
    'BatchToSpaceND',
    'SpaceToBatch',
    'BatchToSpace',
    'Enter',
    'Range',
    'Slice',
    'Placeholder',
    ]
    if op.type in dangerous_list:
      return True
    for o in op.outputs:
      if 'float' not in str(o.dtype):
        return True
      try:
        if len(o.shape) == 0:
          return True
      except ValueError:
        return False
    return False

  def is_input_op(self, op):
    if op.name in self.input_ops:
      return True
    return False


  def is_output_op(self, op):
    if op.name in self.output_ops:
      return True
    return False

  def get_inputs_list(self, op):
    input_list = []
    for ci in op.control_inputs:
      input_list.append(ci.name)
    for i in op.inputs:
      input_list.append(i.op.name)
    return input_list

  def save_subgraph(self, subgraph, save_folder, index, tensors):
    tmp_input_op_list = set()
    tmp_output_op_list = set()
    tmp_multi_input_list = set()
    for isg in subgraph.input_subgraphs:
      for isgop in subgraph.input_subgraphs[isg]:
        tmp_input_op_list.add(isgop)
    for osg in subgraph.output_subgraphs:
      for osgop in subgraph.output_subgraphs[osg]:
        tmp_output_op_list.add(osgop)

    ret_model_info = dict()
    ret_model_info['model_path'] = 'graph_{0}.pb'.format(index)
    ret_sub_inputs = set()
    ret_sub_outputs = set()

    for ori_iname in subgraph.input_ops:
      ori_iop = self.graph.get_operation_by_name(ori_iname)
      assert(len(ori_iop.outputs) == 1)
      ret_sub_inputs.add(ori_iname + ':0')
    for ori_oname in subgraph.output_ops:
      ori_oop = self.graph.get_operation_by_name(ori_oname)
      assert(len(ori_oop.outputs) == 1)
      ret_sub_outputs.add(ori_oname + ':0')

    graph_str = ''
    for i in subgraph.input_subgraphs:
      for iop in subgraph.input_subgraphs[i]:
        op = self.graph.get_operation_by_name(iop)
        if len(op.outputs) > 1:
          tmp_multi_input_list.add(iop)
          idx = len(op.outputs)
          for ii in range(idx):
            newname = iop + '_{0}_sophon_auto'.format(ii)
            newshape = tensors[newname + ':0'].shape
            ret_sub_inputs.add(newname + ':0')
            with tf.Graph().as_default():
              if self.dynamic:
                newop = tf.placeholder(op.outputs[ii].dtype, \
                                        shape=None, name=newname).op
              else:
                newop = tf.placeholder(op.outputs[ii].dtype, \
                                        shape=newshape, name=newname).op
              graph_str = graph_str + node_def_add_node(str(newop.node_def))
        elif len(op.outputs) == 1:
          can_remove, add_list = if_this_input_can_remove(self.graph, iop)
          if can_remove:
            for addname in add_list:
              op_to_add = self.graph.get_operation_by_name(addname)
              op_str = node_def_add_node(str(op_to_add.node_def))
              op_str = node_def_remove_control_inputs(op_str)
              if op_to_add.type == 'Const':
                op_str = node_def_remove_inputs(op_str)
              op_str = node_def_remove_colocations(op_str)
              graph_str = graph_str + op_str
          else:
            ret_sub_inputs.add(iop + ':0')
            newshape = tensors[iop + ':0'].shape
            with tf.Graph().as_default():
              if self.dynamic:
                newop = tf.placeholder(op.outputs[0].dtype, \
                                        shape=None, name=iop).op
              else:
                newop = tf.placeholder(op.outputs[0].dtype, \
                                        shape=newshape, name=iop).op
              graph_str = graph_str + node_def_add_node(str(newop.node_def))
        else:
          with tf.Graph().as_default():
            newop = tf.constant(1.0, name=iop).op
            graph_str = graph_str + node_def_add_node(str(newop.node_def))

    for o in subgraph.ops:
      op = self.graph.get_operation_by_name(o)
      if op.name in tmp_output_op_list and len(op.outputs) > 1:
        with self.graph.as_default():
          for itag,op_output in enumerate(op.outputs):
            newname = op.name + "_{0}_sophon_auto".format(itag)
            newstr = node_def_add_node(str(tf.identity(op_output, \
                                            name=newname).op.node_def))
            graph_str = graph_str + newstr
      op_str = node_def_add_node(str(op.node_def))
      for key in tmp_multi_input_list:
        idx_0 = op_str.find('input: "' + key + ':')
        while idx_0 > 0:
          idx_s = op_str.find('"', idx_0)
          idx_e = op_str.find('"', idx_s + 1)
          idx_i = op_str.find(':', idx_s, idx_e)
          idx = int(op_str[(idx_i + 1) : idx_e])
          new_str = key + '_{0}_sophon_auto'.format(idx)
          op_str = op_str.replace('input: "{0}:{1}"'.format(key, idx), \
                                  'input: "{0}"'.format(new_str))
          idx_0 = op_str.find('input: "' + key + ':')
        idx_0 = op_str.find('input: "' + key + '"')
        while idx_0 > 0:
          new_str = key + '_0_sophon_auto'
          op_str = op_str.replace('input: "'+key+'"', 'input: "'+new_str+'"')
          idx_0 = op_str.find('input: "' + key + '"')
      op_str = node_def_remove_colocations(op_str)
      graph_str = graph_str + op_str
    for og in subgraph.output_subgraphs:
      for oop in subgraph.output_subgraphs[og]:
        op = self.graph.get_operation_by_name(oop)
        if len(op.outputs) > 1:
          num = len(op.outputs)
          for ii in range(num):
            ret_sub_outputs.add(oop + '_{0}_sophon_auto:0'.format(ii))
        elif len(op.outputs) == 1:
          can_remove, _ = if_this_input_can_remove(self.graph, oop)
          if not can_remove:
            ret_sub_outputs.add(oop + ':0')
        else:
          continue
    save_str_to_pb(graph_str, save_folder, 'graph_{0}.pb'.format(index))
    return ret_model_info, list(ret_sub_inputs), list(ret_sub_outputs)

  def infer_output_tensors(self, save_folder, model_info, \
                          sub_inputs, sub_outputs, \
                          tensors):
    model_path = os.path.join(save_folder, model_info['model_path'])
    input_dict = dict()
    for si in sub_inputs:
      input_dict[si] = tensors[si]
    results = tensorflow_infer_outputs(model_path, input_dict, sub_outputs)
#    ret_shapes = []
#    for r in results:
#      ret_shapes.append(r.shape)
    return results

  def destroy(self):
    if hasattr(self, 'sess') and (self.sess is not None):
      self.sess.close()
      self.sess = None
