import sys
sys.path.append("..")
from auto_deploy.api import split, compile, infer
import numpy as np
import cv2
import os
import argparse


def main():
  """Program entry point.
  """
  dynamic = False
  if FLAGS.dynamic != 0:
    dynamic = True
  if os.path.exists(FLAGS.save_dir) and os.path.isfile(FLAGS.save_dir):
    print("{} is a file, please input a directory path.")
    exit(-1)
  if not os.path.exists(FLAGS.save_dir):
    os.mkdir(FLAGS.save_dir)
  save_dir = os.path.abspath(FLAGS.save_dir)
  image = np.expand_dims(cv2.resize(cv2.imread(FLAGS.image_path), (FLAGS.width, FLAGS.height)), 0)
  if FLAGS.model_type == "ssd" or FLAGS.model_type == "frcnn":
    input_tensors = {'image_tensor:0': image}
    output_names = ['num_detections:0', 'detection_classes:0', \
                    'detection_boxes:0', 'detection_scores:0']
  elif FLAGS.model_type == "mrcnn":
    input_tensors = {'image_tensor:0': image}
    output_names = ['num_detections:0', 'detection_classes:0', \
                    'detection_boxes:0', 'detection_scores:0', \
                    'detection_masks:0']
  elif FLAGS.model_type == "deeplab":
    input_tensors = {'ImageTensor:0': image}
    output_names = ['SemanticPredictions:0']
  elif FLAGS.model_type == "test":
    input_tensors = {'Placeholder:0': image, 'Placeholder_1:0': np.array([FLAGS.height, FLAGS.width, 1], dtype=np.float32)}
    output_names = ['resnet_v1_101_3/rois/concat:0',
                    'resnet_v1_101_5/cls_score/BiasAdd:0',
                    'resnet_v1_101_5/cls_prob:0',
                    'add:0']
  else:
    print("Wrong model_type: ", model_type)
    exit(-1)

  if FLAGS.mode & 1 == 1:
    split('tensorflow', input_tensors, save_dir, FLAGS.model_path, params_path=None, outputs=output_names, dynamic=dynamic, layout='NHWC')
    print("Splitting done!\n")

  if FLAGS.mode & 2 == 2:
    compile(save_dir, optimize=1, compare=True, target='BM1682')
    print("Compilation done!\n")

  if FLAGS.mode & 4 == 4:
    results = infer(save_dir, input_tensors)
    print("Inference done!")
    print(results)

if __name__ == "__main__":
  PARSER = argparse.ArgumentParser()
  PARSER.add_argument(
      '--save_dir',
      type=str,
      required=True,
      help='Absolute path of a directory to save splitting information.'
  )
  PARSER.add_argument(
      '--model_path',
      type=str,
      required=True,
      help='Path to model file.'
  )
  PARSER.add_argument(
      '--image_path',
      type=str,
      required=True,
      help='Path to image file.'
  )
  PARSER.add_argument(
      '--model_type',
      type=str,
      required=True,
      help='Options: ssd, frcnn, mrcnn, deeplab'
  )
  PARSER.add_argument(
      '--width',
      type=int,
      required=True,
      help='Width of image.'
  )
  PARSER.add_argument(
      '--height',
      type=int,
      required=True,
      help='height of image.'
  )
  PARSER.add_argument(
      '--dynamic',
      type=int,
      required=True,
      help='If dynamic mode.'
  )
  PARSER.add_argument(
      '--mode',
      type=int,
      required=True,
      help='Options: 1:S,2:C,3:SC,4:R,5:SR,6:CR,7:SCR(S:split,C:compile,R:run)'
  )
  FLAGS, UNPARSED = PARSER.parse_known_args()
  main()
