import sys
sys.path.append("..")
from auto_deploy.api import infer_time
import numpy as np
import cv2
import os
import argparse


def main():
  """Program entry point.
  """
  if os.path.exists(FLAGS.save_dir) and os.path.isfile(FLAGS.save_dir):
    print("{} is a file, please input a directory path.")
    exit(-1)
  if not os.path.exists(FLAGS.save_dir):
    os.mkdir(FLAGS.save_dir)
  save_dir = os.path.abspath(FLAGS.save_dir)
  image = np.expand_dims(cv2.resize(cv2.imread(FLAGS.image_path), (FLAGS.width, FLAGS.height)), 0)
  if FLAGS.model_type == "ssd" or FLAGS.model_type == "frcnn":
    input_tensors = {'image_tensor:0': image}
    output_names = ['num_detections:0', 'detection_classes:0', \
                    'detection_boxes:0', 'detection_scores:0']
  elif FLAGS.model_type == "mrcnn":
    input_tensors = {'image_tensor:0': image}
    output_names = ['num_detections:0', 'detection_classes:0', \
                    'detection_boxes:0', 'detection_scores:0', \
                    'detection_masks:0']
  elif FLAGS.model_type == "deeplab":
    input_tensors = {'ImageTensor:0': image}
    output_names = ['SemanticPredictions:0']
  else:
    print("Wrong model_type: ", model_type)
    exit(-1)

  infer_time(save_dir, input_tensors, os.path.join(save_dir,FLAGS.time_file), mode=FLAGS.time_mode)


if __name__ == "__main__":
  PARSER = argparse.ArgumentParser()
  PARSER.add_argument(
      '--save_dir',
      type=str,
      required=True,
      help='Absolute path of a directory to save splitting information.'
  )
  PARSER.add_argument(
      '--image_path',
      type=str,
      required=True,
      help='Path to image file.'
  )
  PARSER.add_argument(
      '--model_type',
      type=str,
      required=True,
      help='Options: ssd, frcnn, mrcnn, deeplab'
  )
  PARSER.add_argument(
      '--width',
      type=int,
      required=True,
      help='Width of image.'
  )
  PARSER.add_argument(
      '--height',
      type=int,
      required=True,
      help='height of image.'
  )
  PARSER.add_argument(
      '--time_file',
      type=str,
      required=True,
      help='Filename for saving infer time.'
  )
  PARSER.add_argument(
      '--time_mode',
      type=int,
      required=True,
      help='For time test, if only use cpu.'
  )
  FLAGS, UNPARSED = PARSER.parse_known_args()
  main()
