#ifndef FACE_COMMON_HPP_
#define FACE_COMMON_HPP_

#include <opencv2/opencv.hpp>
#include <bmnet.h>
#include <bmblob.h>
#include <bmcnnctx.h>
#include <bmdnn_api.h>
#include <bmcv_api.h>
#include <bmlib_runtime.h>

#include <sys/time.h>
#include <vector>



#include "boost/make_shared.hpp"

// un-comment this if you want to use bm api other than cv functions
#define USING_BM_IMG_API 1
#define FACE_RECG_INFO(fmt, ...) printf("FACE RECG INFO:" fmt, ##__VA_ARGS__)
#define DEBUG
#ifdef DEBUG
    #define FACE_RECG_DBG(fmt, ...) printf("FACE RECG DBG:" fmt, ##__VA_ARGS__)
#else
    #define FACE_RECG_DBG(fmt, ...)
#endif

using namespace std;
using namespace bmcnn;
typedef BMBlob __BLOB;
typedef BMNet __NET;

typedef struct FaceRect {
  float x1;
  float y1;
  float x2;
  float y2;
  float score;
} FaceRect;

typedef struct FacePts {
  float x[5];
  float y[5];
  float score;
} FacePts;

inline void wrapInputLayer(__BLOB* input_layer,
                           std::vector<cv::Mat>* input_channels) {
  int width = input_layer->width();
  int height = input_layer->height();

  float* input_data = input_layer-> mutable_cpu_data();
  for (int i = 0; i < input_layer-> channels(); ++i) {
    cv::Mat channel(height, width, CV_32FC1, input_data);
    input_channels-> push_back(channel);
    input_data += width * height;
  }
}

inline long long timestamp(void) {
    long long t = 0;
    struct timeval tv;
    gettimeofday(&tv, NULL);
    t = tv.tv_sec * 1000000;
    t += tv.tv_usec;
    return t;
}

#endif  // FACE_COMMON_HPP_
