#ifndef FACE_DETECT_HPP_
#define FACE_DETECT_HPP_
// #include "boost/make_shared.hpp"
#include <face_common.hpp>

#include <vector>
// un-comment this if you want to use bm api other than cv functions
#define USING_BM_IMG_API 1
#define USING_DEVICE_MEM 1

#define RESIZED_WIDTH 711
#define RESIZED_HEIGHT 400

#define SAVE_DET_FACE_FILE 0
#define FRAME_NUMBER_TO_SAVE 85

#define DEBUG_EXTRACT 1

#ifdef SOC_MODE
#define USE_MAPPED_OUTPUT 1
#endif


class FaceDetector {
 public:
        explicit FaceDetector(bmcnn_ctx_t bmcnn_ctx);
        void detect(bm_device_mem_t *input_y, int ystride,
                    bm_device_mem_t *input_uv, int uvstride,
                    int width, int height,
                    const float threshold,
                    std::vector<FaceRect>& faceRects);
        void detect(bm_device_mem_t *input_bgr,
                    int width, int height,
                    const float threshold,
                    std::vector<FaceRect>& faceRects);
        void detect(cv::Mat &image, const float threshold,
                    std::vector<FaceRect>& faceRects);

 private:
        void do_detect(bm_device_mem_t *dmem,
                       int width, int height,
                       const float threshold,
                       std::vector<FaceRect> &faceRects);
        void calculatePyramidScale(cv::Mat image, bm_device_mem_t *output_dmem);
        void calculatePyramidScale(bm_device_mem_t *input_bgr,
                                   int width, int height,
                                   bm_device_mem_t *output_dmem);
        void calculatePyramidScale(bm_device_mem_t *input_y, int ystride,
                                   bm_device_mem_t *input_uv, int uvstride,
                                   int width, int height,
                                   bm_device_mem_t *output_dmem);
        void generateProposals(const float* scores, const float* bbox_deltas,
                const float scale_factor, const int feat_factor,
                const int feat_w, const int feat_h,
                const int width, const int height,
                std::vector<FaceRect>& proposals);
        void nms(const std::vector<FaceRect>& proposals,
                std::vector<FaceRect>& nmsProposals);
        void initParameters(void);

        boost::shared_ptr<__NET> net_;

        double target_size_;
        double max_size_;
        double im_scale_;
        float nms_threshold_;
        float base_threshold_;
        std::vector<float> anchor_ratios_;
        std::vector<float> anchor_scales_;
        int per_nms_topn_;
        int base_size_;
        int min_size_;
        int feat_stride_;
        int anchor_num_;
};

#endif  // FACE_DETECT_HPP_
