#include "face_detect.hpp"

#include <sys/time.h>

#ifdef USING_BM_IMG_API
#include <bmdnn_api.h>
#include <bmcv_api.h>
// #include <bmruntime.h>
#include <bmruntime_interface.h>
#include <bmblob.h>
#include <bmlib_runtime.h>
#include <cstring>
#include <vector>
#include <algorithm>
#include <iostream>
#include <cmath>
bm_handle_t handle;
#endif

extern int debugmode;
#define DEBUG_DETECT 0
#define SQUEEZENET

using namespace std;
void FaceDetector::initParameters(void) {
    target_size_ = 400;
    max_size_ = 800;
    nms_threshold_ = 0.3;
    base_threshold_ = 0.05;
    per_nms_topn_ = 1000;
    base_size_ = 16;
    min_size_ = 0;
    feat_stride_ = 8;

    int ratio_size = 1;

    anchor_ratios_.push_back(1.f);

    anchor_scales_.push_back(1);
    anchor_scales_.push_back(2);
    anchor_num_ = ratio_size * 2;
}

FaceDetector::FaceDetector(bmcnn_ctx_t bmcnn_ctx) {
#if defined(USING_BM_IMG_API)
    handle = (bm_handle_t)bmrt_get_bm_handle(bmcnn_ctx);
#endif
    std::cout << "Loading nets ..." << std::endl;
    net_.reset(new __NET(bmcnn_ctx, "SqueezeNet"));

    initParameters();
}

// input cv mat img is byte bgrbgrbgr format
void FaceDetector::calculatePyramidScale(cv::Mat image,
                                         bm_device_mem_t *output_dmem) {
#if defined(SSH)
    float scalar[3] = {103.94f, 111.78f, 123.68f};
#elif defined(SQUEEZENET)
    float scalar[3] = {104.0f, 117.0f, 123.0f};
#else
    float scalar[3] = {0.0f};
    std::cout << "No detection net specified" <<endl;
    exit(-1);
#endif

        std::vector<cv::Mat> input_channels;
        // cv::Mat target_img = cvCreateMat(image.rows,image.cols,CV_32FC3);
        cv::Mat target_img;
    target_img.create(image.rows, image.cols, CV_32FC3);
        unsigned char *input_data = target_img.data;
        for (int i = 0; i < 3; ++i) {
                cv::Mat channel(image.rows, image.cols, CV_8UC1, input_data);
                input_channels.push_back(channel);
                input_data += image.rows * image.cols;
        }
        cv::split(image, input_channels);

        int height = target_img.rows;
        int width = target_img.cols;

    int im_size_min = std::min(height, width);
    int im_size_max = std::max(height, width);
    im_scale_ = target_size_ / im_size_min;

    std::cout << "width x height " << width << "x"<< height <<std::endl;
    bmcv_image input;
    input.color_space = COLOR_RGB;
    input.data_format = DATA_TYPE_BYTE;
    input.image_format = BGR;
    input.image_width = width;
    input.image_height = height;
    input.data[0] = bm_mem_from_system(target_img.data);
    input.stride[0] = width;

    bmcv_image output;
    output.color_space = COLOR_RGB;
    output.data_format = DATA_TYPE_FLOAT;
    output.image_format = BGR;
    output.image_width = RESIZED_WIDTH;
    output.image_height = RESIZED_HEIGHT;
    output.data[0] = *output_dmem;
    output.stride[0] = RESIZED_WIDTH;
    bmcv_img_scale(handle,
            input,
            1,
            0, 0, 0, 0, 0,  /* do not crop */
            1, 0, 0, 0,
            1, 1.0f, -scalar[0], 1.0f, -scalar[1], 1.0f, -scalar[2],
            output);
}


// here the input device mem is pointing to
// byte type bgr device memory converted from ffmpeg output
void FaceDetector::calculatePyramidScale(bm_device_mem_t *input_bgr,
                                         int width,
                                         int height,
                                         bm_device_mem_t *output_dmem) {
#if defined(SSH)
    float scalar[3] = {103.94f, 111.78f, 123.68f};
#elif defined(SQUEEZENET)
    float scalar[3] = {104.0f, 117.0f, 123.0f};
#else
    float scalar[3] = {0.0f};
    std::cout << "No detection net specified" << endl;
    exit(-1);
#endif

    int im_size_min = std::min(height, width);
    int im_size_max = std::max(height, width);
    im_scale_ = target_size_ / im_size_min;

    bmcv_image input;
    input.color_space = COLOR_RGB;
    input.data_format = DATA_TYPE_BYTE;
    input.image_format = BGR;
    input.image_width = width;
    input.image_height = height;
    input.data[0] = *input_bgr;
    input.stride[0] = width;

    bmcv_image output;
    output.color_space = COLOR_RGB;
    output.data_format = DATA_TYPE_FLOAT;
    output.image_format = BGR;
    output.image_width = RESIZED_WIDTH;
    output.image_height = RESIZED_HEIGHT;
    output.data[0] = *output_dmem;
    output.stride[0] = RESIZED_WIDTH;

    bmcv_img_scale(handle,
            input,
            1,
            0, 0, 0, 0, 0,  /* do not crop */
            1, 0, 0, 0,
            1, 1.0f, -scalar[0], 1.0f, -scalar[1], 1.0f, -scalar[2],
            output);
}

// here the input device mem is pointing to ffmpeg output, in nv12 format
void FaceDetector::calculatePyramidScale(bm_device_mem_t *input_y,
                                         int ystride,
                                         bm_device_mem_t *input_uv,
                                         int uvstride,
                                         int width, int height,
                                         bm_device_mem_t *output_dmem) {
#if defined(SSH)
    float scalar[3] = {103.94f, 111.78f, 123.68f};
#elif defined(SQUEEZENET)
    float scalar[3] = {104.0f, 117.0f, 123.0f};
#else
    float scalar[3] = {0.0f};
    std::cout << "No detection net specified" << endl;
    exit(-1);
#endif

    int im_size_min = std::min(height, width);
    int im_size_max = std::max(height, width);
    im_scale_ = target_size_ / im_size_min;

    bm_device_mem_t dmem_bgr;
    memset(&dmem_bgr, 0, sizeof(bm_device_mem_t));
    int ret = bm_malloc_device_byte(handle, &dmem_bgr, (width*height*3+3)/4*4);
    if (ret != BM_SUCCESS) {
        printf("malloc dmem fail!\n");
        return;
    }
    unsigned long long t1 = timestamp();
    bmcv_image input;
    input.color_space = COLOR_YUV;
    input.data_format = DATA_TYPE_BYTE;
    input.image_format = NV12;
    input.data[0] = *input_y;
    input.stride[0] = ystride;
    input.data[1] = *input_uv;
    input.stride[1] = uvstride;
    input.image_width = width;
    input.image_height = height;

    bmcv_image output = input;
    output.color_space = COLOR_RGB;
    output.image_format = BGR;
    output.data[0] = dmem_bgr;
    output.stride[0] = width;

    bm_img_yuv2bgr(handle,
            input, output);
    unsigned long long t2 = timestamp();

    bmcv_image out = output;
    out.image_height = RESIZED_HEIGHT;
    out.image_width = RESIZED_WIDTH;
    out.data_format = DATA_TYPE_FLOAT;
    out.data[0] = *output_dmem;
    out.stride[0] = RESIZED_WIDTH;
    bmcv_img_scale(handle,
            output,
            1,
            0, 0, 0, 0, 0,  /* do not crop */
            1, 0, 0, 0,
            1, 1.0f, -scalar[0], 1.0f, -scalar[1], 1.0f, -scalar[2],
            out);
    unsigned long long t3 = timestamp();
    if (debugmode && DEBUG_DETECT) {
        printf("BENCH yuv2bgr cost %lld us resize cost %lld us\n",
                t2-t1, t3-t2);
    }

    bm_free_device(handle, dmem_bgr);
}

inline bool compareBBox(const FaceRect& a, const FaceRect& b) {
    return a.score > b.score;
}

void FaceDetector::generateProposals(
        const float* scores, const float* bbox_deltas,
        const float scale_factor,
        const int feat_factor, const int feat_w,
        const int feat_h, const int width,
        const int height, std::vector<FaceRect>& proposals) {
    std::vector<FaceRect> m_proposals;
    float anchor_cx = (base_size_ - 1) * 0.5;
    float anchor_cy = (base_size_ - 1) * 0.5;
    int feat_stride = feat_stride_ * feat_factor;

    for (unsigned int s = 0; s < anchor_scales_.size(); ++s) {
        float scale = anchor_scales_[s] * scale_factor;
        for (int h = 0; h < feat_h; ++h) {
            for (int w = 0; w < feat_w; ++w) {
                int delta_index = h * feat_w + w;
                FaceRect facerect;
                facerect.score = scores[s * feat_w * feat_h + delta_index];
                if (facerect.score <= base_threshold_) continue;
                float anchor_size = scale * base_size_;
                float bbox_x1 = anchor_cx - (anchor_size - 1) * 0.5 +
                                w * feat_stride;
                float bbox_y1 = anchor_cy - (anchor_size - 1) * 0.5 +
                                h * feat_stride;
                float bbox_x2 = anchor_cx + (anchor_size - 1) * 0.5 +
                                w * feat_stride;
                float bbox_y2 = anchor_cy + (anchor_size - 1) * 0.5 +
                                h * feat_stride;

                float bbox_w = bbox_x2 - bbox_x1 + 1;
                float bbox_h = bbox_y2 - bbox_y1 + 1;
                float bbox_cx = bbox_x1 + 0.5 * bbox_w;
                float bbox_cy = bbox_y1 + 0.5 * bbox_h;
                float dx = bbox_deltas[(s * 4 + 0) *
                                       feat_h * feat_w + delta_index];
                float dy = bbox_deltas[(s * 4 + 1) *
                                       feat_h * feat_w + delta_index];
                float dw = bbox_deltas[(s * 4 + 2) *
                                       feat_h * feat_w + delta_index];
                float dh = bbox_deltas[(s * 4 + 3) *
                                       feat_h * feat_w + delta_index];
                float pred_cx = dx * bbox_w + bbox_cx;
                float pred_cy = dy * bbox_h + bbox_cy;
                float pred_w = std::exp(dw) * bbox_w;
                float pred_h = std::exp(dh) * bbox_h;
                facerect.x1 = std::max(std::min(static_cast<double>(width - 1),
                            (pred_cx - 0.5 * pred_w) / im_scale_),
                        0.0);
                facerect.y1 = std::max(std::min(static_cast<double>(height - 1),
                            (pred_cy - 0.5 * pred_h) / im_scale_),
                        0.0);
                facerect.x2 = std::max(std::min(static_cast<double>(width - 1),
                            (pred_cx + 0.5 * pred_w) / im_scale_),
                        0.0);
                facerect.y2 = std::max(std::min(static_cast<double>(height - 1),
                            (pred_cy + 0.5 * pred_h) / im_scale_),
                        0.0);
                if ((facerect.x2 - facerect.x1 + 1 < min_size_) ||
                        (facerect.y2 - facerect.y1 + 1 < min_size_))
                    continue;
                m_proposals.push_back(facerect);
            }
        }
    }
    std::sort(m_proposals.begin(), m_proposals.end(), compareBBox);

    int keep = m_proposals.size();
    if (per_nms_topn_ < keep) keep = per_nms_topn_;

    if (keep > 0) {
        proposals.insert(proposals.end(), m_proposals.begin(),
                m_proposals.begin() + keep);
    }
}

void FaceDetector::nms(const std::vector<FaceRect>& proposals,
        std::vector<FaceRect>& nmsProposals) {
    if (proposals.empty()) {
        nmsProposals.clear();
        return;
    }
    std::vector<FaceRect> bboxes = proposals;
    std::sort(bboxes.begin(), bboxes.end(), compareBBox);

    int select_idx = 0;
    int num_bbox = bboxes.size();
    std::vector<int> mask_merged(num_bbox, 0);
    bool all_merged = false;
    while (!all_merged) {
        while (select_idx < num_bbox && 1 ==
                mask_merged[select_idx]) {
                    ++select_idx;
        }

        if (select_idx == num_bbox) {
            all_merged = true;
            continue;
        }
        nmsProposals.push_back(bboxes[select_idx]);
        mask_merged[select_idx] = 1;
        FaceRect select_bbox = bboxes[select_idx];
        float area1 = (select_bbox.x2 - select_bbox.x1 + 1) *
            (select_bbox.y2 - select_bbox.y1 + 1);
        ++select_idx;
        for (int i = select_idx; i < num_bbox; ++i) {
            if (mask_merged[i] == 1) continue;
            FaceRect& bbox_i = bboxes[i];
            float x = std::max(select_bbox.x1, bbox_i.x1);
            float y = std::max(select_bbox.y1, bbox_i.y1);
            float w = std::min(select_bbox.x2, bbox_i.x2) - x + 1;
            float h = std::min(select_bbox.y2, bbox_i.y2) - y + 1;
            if (w <= 0 || h <= 0) continue;
            float area2 = (bbox_i.x2 - bbox_i.x1 + 1) *
                          (bbox_i.y2 - bbox_i.y1 + 1);
            float area_intersect = w * h;
            // Union method
            if (area_intersect / (area1 + area2 - area_intersect)
                > nms_threshold_)
                mask_merged[i] = 1;
        }
    }
}

void FaceDetector::detect(cv::Mat &image, const float threshold,
                          std::vector<FaceRect>& faceRects) {
    int net_h = RESIZED_HEIGHT;
    int net_w = RESIZED_WIDTH;
    int net_c = 3;
    int net_b = 1;

    boost::shared_ptr<__BLOB> input_blob = net_-> blob_by_name("data");

    input_blob->Reshape(net_b, net_c, net_h, net_w);
    // if not using bm image api, above code will split/copy data to blob
    // we have the data in planar layout, so we just copy it into blob
    // we should find ways to set device desc directly.
    bm_device_mem_t *dmem = input_blob.get()->mutable_dev_mem();

    calculatePyramidScale(image, dmem);

    do_detect(dmem, image.cols, image.rows, threshold, faceRects);
}

void FaceDetector::detect(bm_device_mem_t *input_bgr, int width,
                          int height, const float threshold,
                           std::vector<FaceRect>& faceRects) {
    int net_h = RESIZED_HEIGHT;
    int net_w = RESIZED_WIDTH;
    int net_c = 3;
    int net_b = 1;

    boost::shared_ptr<__BLOB> input_blob = net_-> blob_by_name("data");

    input_blob->Reshape(net_b, net_c, net_h, net_w);
    // if not using bm image api, above code will split/copy data to blob
    // we have the data in planar layout, so we just copy it into blob
    // we should find ways to set device desc directly.
    bm_device_mem_t *dmem = input_blob.get()->mutable_dev_mem();

    calculatePyramidScale(input_bgr, width, height, dmem);

    do_detect(dmem, width, height, threshold, faceRects);
}

void FaceDetector::detect(bm_device_mem_t *input_y, int ystride,
                          bm_device_mem_t *input_uv, int uvstride,
                          int width, int height,  const float threshold,
                          std::vector<FaceRect>& faceRects) {
    int net_h = RESIZED_HEIGHT;
    int net_w = RESIZED_WIDTH;
    int net_c = 3;
    int net_b = 1;

    boost::shared_ptr<__BLOB> input_blob = net_-> blob_by_name("data");

    input_blob->Reshape(net_b, net_c, net_h, net_w);
    // if not using bm image api, above code will split/copy data to blob
    // we have the data in planar layout, so we just copy it into blob
    // we should find ways to set device desc directly.
    bm_device_mem_t *dmem = input_blob.get()->mutable_dev_mem();

    calculatePyramidScale(input_y, ystride, input_uv,
                          uvstride, width, height, dmem);

    do_detect(dmem, width, height, threshold, faceRects);
}

void FaceDetector::do_detect(bm_device_mem_t *dmem,
                             int width,
                             int height,
                             const float threshold,
                             std::vector<FaceRect> &faceRects ) {
    unsigned long long t1 = timestamp();
    net_->Forward();
    bm_flush(handle);
    unsigned long long t2 = timestamp();
    printf("time to detect %lld us\n", t2-t1);

    std::vector<FaceRect> proposals;

    unsigned long long t3 = timestamp();
    __BLOB* m3_cls_tensor =
        net_->blob_by_name("m3@ssh_cls_prob_reshape_output").get();
    __BLOB* m3_bbox_tensor =
        net_->blob_by_name("m3@ssh_bbox_pred_output").get();
    int m3_w = m3_cls_tensor->width();
    int m3_h = m3_cls_tensor->height();
#ifdef USE_MAPPED_OUTPUT
    const float* m3_scores =
        (const float *)m3_cls_tensor->mutable_mapped_output();
#else
    const float* m3_scores = m3_cls_tensor->cpu_data();
#endif
    m3_scores += anchor_num_ * m3_w * m3_h;
#ifdef USE_MAPPED_OUTPUT
    const float* m3_bbox_deltas =
        (const float *)m3_bbox_tensor->mutable_mapped_output();
#else
    const float* m3_bbox_deltas = m3_bbox_tensor->cpu_data();
#endif
    generateProposals(m3_scores, m3_bbox_deltas, 16.0, 4, m3_w,
            m3_h, width, height, proposals);

    __BLOB* m2_cls_tensor =
        net_->blob_by_name("m2@ssh_cls_prob_reshape_output").get();
    __BLOB* m2_bbox_tensor =
        net_->blob_by_name("m2@ssh_bbox_pred_output").get();
    int m2_w = m2_cls_tensor->width();
    int m2_h = m2_cls_tensor->height();
#ifdef USE_MAPPED_OUTPUT
    const float* m2_scores =
        (const float *)m2_cls_tensor->mutable_mapped_output();
#else
    const float* m2_scores = m2_cls_tensor->cpu_data();
#endif
    m2_scores += anchor_num_ * m2_w * m2_h;
#ifdef USE_MAPPED_OUTPUT
    const float* m2_bbox_deltas =
        (const float *)m2_bbox_tensor->mutable_mapped_output();
#else
    const float* m2_bbox_deltas = m2_bbox_tensor->cpu_data();
#endif
    generateProposals(m2_scores, m2_bbox_deltas, 4.0, 2, m2_w,
            m2_h, width, height, proposals);

    __BLOB* m1_cls_tensor =
        net_->blob_by_name("m1@ssh_cls_prob_reshape_output").get();
    __BLOB* m1_bbox_tensor =
        net_->blob_by_name("m1@ssh_bbox_pred_output").get();
    int m1_w = m1_cls_tensor->width();
    int m1_h = m1_cls_tensor->height();
#ifdef USE_MAPPED_OUTPUT
    const float* m1_scores =
        (const float *)m1_cls_tensor->mutable_mapped_output();
#else
    const float* m1_scores = m1_cls_tensor->cpu_data();
#endif
    m1_scores += anchor_num_ * m1_w * m1_h;
#ifdef USE_MAPPED_OUTPUT
    const float* m1_bbox_deltas =
        (const float *)m1_bbox_tensor->mutable_mapped_output();
#else
    const float* m1_bbox_deltas = m1_bbox_tensor->cpu_data();
#endif
    generateProposals(m1_scores, m1_bbox_deltas, 1.0, 1, m1_w,
            m1_h, width, height, proposals);

    unsigned long long t4 = timestamp();
    std::vector<FaceRect> nmsProposals;
    nms(proposals, nmsProposals);
    unsigned long long t5 = timestamp();

    printf("time to genproposal %lld nms %lld us\n", t4-t3, t5-t4);

    faceRects.clear();
    for (unsigned int i = 0; i < nmsProposals.size(); ++i) {
        FaceRect rect = nmsProposals[i];
        if (debugmode && DEBUG_DETECT) {
            std::cout << "nmsProposals " << i << " bboxes" << std::endl;
        }

        if (rect.score >= threshold) {
            faceRects.push_back(rect);
            if (debugmode && DEBUG_DETECT) {
                std::cout << "final predict " << i
                          << " bboxes " << rect.score
                          << " " << rect.x1
                          << "," << rect.y1 << std::endl;
            }
        }
    }
    std::cout << "final predict " << faceRects.size() << " bboxes" << std::endl;
}

