#include "face_extract.hpp"

#if USE_CAFFE
FaceExtract::FaceExtract(const std::string& proto_file,
                         const std::string& model_file) {
  net_.reset(new __NET(proto_file, TEST));
  net_-> CopyTrainedLayersFrom(model_file);
  int height = 112;
  int width = 112;
  input_geometry_ = cv::Size(width, height);
}
#else
FaceExtract::FaceExtract(bmcnn_ctx_t bmcnn_ctx) {
  net_.reset(new __NET(bmcnn_ctx, "r50i-model"));
  int height = 112;
  int width = 112;
  input_geometry_ = cv::Size(width, height);
}
#endif

void FaceExtract::extract(const cv::Mat& image,
                                std::vector<float>& feature) {
  boost::shared_ptr<__BLOB> input_blob = net_-> blob_by_name("data");
  input_blob->Reshape(1, 3, input_geometry_.height, input_geometry_.width);
  cv::Mat sample;
  cv::Mat resized;
  image.copyTo(sample);
  cvtColor(sample, sample, CV_BGR2RGB);

#ifdef DEBUG_EXTRACT
  std::cout << "sample dims " << sample.dims
            << " rows " << sample.rows
            << " cols " << sample.cols
            << " channels " << sample.channels() << std::endl;
#endif
  cv::resize(sample, resized, input_geometry_, 0, 0, cv::INTER_NEAREST);
  resized.convertTo(resized, CV_32FC3, 0.0078125, -127.5 * 0.0078125);
#ifdef DEBUG_EXTRACT
  std::cout << "resized dims " << resized.dims
            << " rows " << resized.rows
            << " cols " << resized.cols
            << " channels " << resized.channels()  << std::endl;
#endif

  std::vector<cv::Mat> input_channels;
  wrapInputLayer(input_blob.get(), &input_channels);
  cv::split(resized, input_channels);

  net_->Forward();

  __BLOB* output_tensor =
      net_->blob_by_name("fc1").get();
  const float* data = output_tensor->cpu_data();
  const float* begin = data;
  const float* end = begin + output_tensor->channels();
  std::vector<float> temp = std::vector<float>(begin, end);
  feature.insert(feature.begin(), temp.begin(), temp.end());
}
