// #include "face_common.hpp"
#include "face_landmark.hpp"
#include <bmcv_api.h>
// #include <bmdnn_api.h>
#include <bmruntime_interface.h>
#include <bmblob.h>
#include <bmlib_runtime.h>
bm_handle_t handle1;
#if USE_CAFFE
FaceLandmark::FaceLandmark(const std::string &proto_file,
                           const std::string &model_file) {
  net_.reset(new __NET(proto_file, TEST));
  net_->CopyTrainedLayersFrom(model_file);
  int height = 48;
  int width = 48;
  input_geometry_ = cv::Size(width, height);
}
#else
FaceLandmark::FaceLandmark(bmcnn_ctx_t bmcnn_ctx) {
  std::cout << "Loading nets ..." << std::endl;
  net_.reset(new __NET(bmcnn_ctx, "LNet_v1"));
  int height = 48;
  int width = 48;
  input_geometry_ = cv::Size(width, height);
  handle1 = (bm_handle_t)bmrt_get_bm_handle(bmcnn_ctx);
}
#endif
void FaceLandmark::landmark(bmcv_image input_bgr, FacePts &facePt) {
  boost::shared_ptr<__BLOB> input_blob = net_->blob_by_name("data");
  input_blob->Reshape(1, 3, input_geometry_.height, input_geometry_.width);
  bm_device_mem_t *dmem = input_blob.get()->mutable_dev_mem();
  bmcv_image output_bgr;
  output_bgr.color_space = COLOR_RGB;
  output_bgr.image_format = BGR;
  output_bgr.image_height = 48;
  output_bgr.image_width = 48;
  output_bgr.data[0] = *dmem;
  output_bgr.stride[0] = 48;
  output_bgr.data_format = DATA_TYPE_FLOAT;

  bmcv_img_scale(
      handle1,
      input_bgr,
      1,
      0, 0, 0, 0, 0,
      1, 0, 0, 0,
      1,
      0.0078125, -127.5 * 0.0078125,
      0.0078125, -127.5 * 0.0078125,
      0.0078125, -127.5 * 0.0078125,
      output_bgr);
  net_->Forward();

  int width = input_bgr.image_width;
  int height = input_bgr.image_height;
  __BLOB *prob_tensor =
      net_->blob_by_name("prob1").get();
  __BLOB *points_tensor =
      net_->blob_by_name("conv6-2").get();
  const float *prob_data = prob_tensor->cpu_data();
  const float *points_data = points_tensor->cpu_data();
  for (int j = 0; j < 5; ++j) {
    facePt.x[j] = width * points_data[2 * j] - 1;
    facePt.y[j] = height * points_data[2 * j + 1] - 1;
  }
  facePt.score = prob_data[0];
}

void FaceLandmark::landmark(const cv::Mat &image, FacePts &facePt) {
  boost::shared_ptr<__BLOB> input_blob = net_->blob_by_name("data");
  input_blob->Reshape(1, 3, input_geometry_.height, input_geometry_.width);
  cv::Mat resized;
  cv::resize(image, resized, input_geometry_, 0, 0, cv::INTER_LINEAR);
  resized.convertTo(resized, CV_32FC3, 0.0078125, -127.5 * 0.0078125);

  std::vector<cv::Mat> input_channels;
  // only support batch_size == 1 by now
  wrapInputLayer(input_blob.get(), &input_channels);
  cv::split(resized, input_channels);

  net_->Forward();

  int width = image.cols;
  int height = image.rows;
  __BLOB *prob_tensor =
      net_->blob_by_name("prob1").get();
  __BLOB *points_tensor =
      net_->blob_by_name("conv6-2").get();
  const float *prob_data = prob_tensor->cpu_data();
  const float *points_data = points_tensor->cpu_data();
  for (int j = 0; j < 5; ++j) {
    facePt.x[j] = width * points_data[2 * j] - 1;
    facePt.y[j] = height * points_data[2 * j + 1] - 1;
  }
  facePt.score = prob_data[0];
}
