#ifndef SAMPLES_CPP_BMNNSDK_RUN_INCEPTIONV1_INCLUDE_UTIL_HPP_
#define SAMPLES_CPP_BMNNSDK_RUN_INCEPTIONV1_INCLUDE_UTIL_HPP_
#include <opencv2/opencv.hpp>
#include <opencv2/core/core.hpp>
#include <bmdnn_api.h>
#include <bmruntime_interface.h>
#include <stdio.h>
#include <iostream>
#include <algorithm>
#include <string>
#include <set>
#include <vector>
#include <cmath>
#include<numeric>

using std::cout;
using std::endl;
using std::max;
using std::min;
using std::pair;
using std::set;
using std::string;
using std::vector;

struct bmrt_tensor_shape_info {
    int dims;
    int shape[BM_MAX_DIMS_NUM];
};

struct bmrt_net_info {
    string net_name;
    int itensor_num;
    int *input_shapes;
    int *input_dims;  // for call launch_shape
    bmrt_tensor_shape_info *input_tensor_info;
    bmrt_tensor_shape_info *input_tensor_max_info;
    float **input_tensor_sys;
    bm_device_mem_t *input_mem_ping;
    bm_device_mem_t *input_mem_pong;
    int otensor_num;
    bmrt_tensor_shape_info *output_tensor_info;
    bmrt_tensor_shape_info *output_tensor_max_info;
    float **output_tensor_sys;
    bm_device_mem_t *output_mem_ping;
    bm_device_mem_t *output_mem_pong;
};

struct bmrt_info {
    void *p_bmrt;
    int net_num;
    bmrt_net_info *netinfo;
};

int bmrt_calcu_tensor_size(bmrt_tensor_shape_info *tensor_shape_info) {
    int tensor_size = 1;
    for (int i = 0; i < tensor_shape_info->dims; i++) {
        tensor_size *= tensor_shape_info->shape[i];
    }
    return tensor_size;
}

const set<string> &bmrt_get_input_tensor_set(void *p_bmrt, int net_idx,
                                             const string &net_name,
                                             set<string> &g_input_tensors) {
    int input_num;
    const char **input_names = NULL;
    bmrt_get_input_tensor(p_bmrt,
                          net_idx,
                          net_name.c_str(),
                          &input_num,
                          &input_names);
    g_input_tensors.clear();
    for (int i = 0; i < input_num; i++) {
        string tensor_name = input_names[i];
        g_input_tensors.insert(tensor_name);
    }
    free(input_names);
    return g_input_tensors;
}

const set<string> &bmrt_get_output_tensor_set(void *p_bmrt, int net_idx,
                                              const string &net_name,
                                              set<string> &g_output_tensors) {
    int output_num;
    const char **output_names = NULL;
    bmrt_get_output_tensor(p_bmrt, net_idx,
                           net_name.c_str(),
                           &output_num, &output_names);
    g_output_tensors.clear();
    for (int i = 0; i < output_num; i++) {
        string tensor_name = output_names[i];
        g_output_tensors.insert(tensor_name);
    }
    free(output_names);
    return g_output_tensors;
}

/* get all info from p_bmrt and alloc device and system memory  */
void bmrt_init_net_info(void *p_bmrt, struct bmrt_info &bmrt_info) {
    /*bmrt_info.net_num = p_bmrt->net_num;  can not get protected data!*/
    bmrt_info.net_num = bmrt_get_network_number(p_bmrt);
    bmrt_info.netinfo = NULL;
    bmrt_info.netinfo = new bmrt_net_info[bmrt_info.net_num];

    /* 1. set all net's name */
    const char **net_names;
    bmrt_get_network_names(p_bmrt, &net_names);
    /* 2. alloc all input/output tensor's information buffer */
    for (int netidx = 0; netidx < bmrt_info.net_num; netidx++) {
        bmrt_net_info *pnetinfo = &bmrt_info.netinfo[netidx];
        set<string> input_tensors, output_tensors;
        bmrt_get_input_tensor_set(p_bmrt,
                                  netidx,
                                  std::string(),
                                  input_tensors);
        bmrt_get_output_tensor_set(p_bmrt,
                                   netidx,
                                   std::string(),
                                   output_tensors);
        pnetinfo->net_name = net_names[netidx];
        pnetinfo->itensor_num = input_tensors.size();
        for (int i = 0; i < pnetinfo->itensor_num; i++) {
            pnetinfo->input_tensor_info =
                new bmrt_tensor_shape_info[pnetinfo->itensor_num];
            pnetinfo->input_tensor_max_info =
                new bmrt_tensor_shape_info[pnetinfo->itensor_num];
            pnetinfo->input_shapes =
                new int[pnetinfo->itensor_num * BM_MAX_DIMS_NUM];
            pnetinfo->input_dims =
                new int[pnetinfo->itensor_num];
        }
        pnetinfo->otensor_num = output_tensors.size();
        for (int i = 0; i < pnetinfo->otensor_num; i++) {
            pnetinfo->output_tensor_info =
                new bmrt_tensor_shape_info[pnetinfo->otensor_num];
            pnetinfo->output_tensor_max_info =
                new bmrt_tensor_shape_info[pnetinfo->otensor_num];
        }
    }
    free(net_names);
    /* 3. fill all input/output tensor's information to buffer */
    for (int netidx = 0; netidx < bmrt_info.net_num; netidx++) {
        bmrt_net_info *pnetinfo = &bmrt_info.netinfo[netidx];
        set<string> input_tensors, output_tensors;
        bmrt_get_input_tensor_set(p_bmrt, netidx,
                                  std::string(), input_tensors);
        bmrt_get_output_tensor_set(p_bmrt, netidx,
                                  std::string(), output_tensors);
        int tensor_idx = 0;  // one net may have many in/out tensors
        int shape_offset = 0;
        /*  get input tensor's infor*/
        for (set<string>::iterator iter = input_tensors.begin();
             iter != input_tensors.end(); ++iter) {
            pnetinfo->input_tensor_max_info[tensor_idx].dims =
                bmrt_get_input_blob_max_shape(p_bmrt, (*iter).c_str(),
                    netidx, NULL,
                    pnetinfo->input_tensor_max_info[tensor_idx].shape);
            pnetinfo->input_dims[tensor_idx] =
                pnetinfo->input_tensor_max_info[tensor_idx].dims;
            for (int i = 0; i < pnetinfo->input_dims[tensor_idx]; i++) {
                pnetinfo->input_shapes[shape_offset + i] =
                    pnetinfo->input_tensor_max_info[tensor_idx].shape[i];
            }
            shape_offset += pnetinfo->input_dims[tensor_idx];
            tensor_idx++;
        }

        /*  get output tensor's info*/
        tensor_idx = 0;
        for (set<string>::const_iterator iter = output_tensors.begin();
             iter != output_tensors.end(); ++iter) {
            pnetinfo->output_tensor_max_info[tensor_idx].dims =
                bmrt_get_output_blob_max_shape(p_bmrt, (*iter).c_str(),
                    netidx, NULL,
                    pnetinfo->output_tensor_max_info[tensor_idx].shape);
            pnetinfo->output_tensor_info[tensor_idx].dims =
                    pnetinfo->output_tensor_max_info[tensor_idx].dims;
            for (int i = 0;
                    i < pnetinfo->output_tensor_info[tensor_idx].dims; i++) {
                pnetinfo->output_tensor_info[tensor_idx].shape[i] =
                    pnetinfo->output_tensor_max_info[tensor_idx].shape[i];
            }
            tensor_idx++;
        }
    }
}

void bmrt_print_shape(const char *prefix,
                      int idx,
                      const char *name,
                      int *shape,
                      int dim) {
    if (prefix) {
        printf("%s ", prefix);
    }
    if (idx >= 0) {
        printf("idx=%02d ", idx);
    }
    printf("shape=[ ");
    for (int i = 0; i < dim; i++) {
        printf("%4d ", shape[i]);
    }
    printf("] ");
    if (name) {
        printf("name=%s", name);
    }
    printf("\n");
}

void bmrt_print_net_info(struct bmrt_info &bmrt_info) {
    cout << "this test has " << bmrt_info.net_num << " nets" << endl
         << endl;

    for (int i = 0; i < bmrt_info.net_num; i++) {
        bmrt_net_info *pnetinfo = &bmrt_info.netinfo[i];
        cout << "net" << i << "'s name: '" << pnetinfo->net_name << "'" << endl;
        const char **tensor_names;
        int tensor_num_;
        int shape_offset = 0;
        bmrt_get_input_tensor(bmrt_info.p_bmrt,
                              i,
                              NULL,
                              &tensor_num_,
                              &tensor_names);
        cout << "this net has " << pnetinfo->itensor_num
             << " input tensor" << endl;
        for (int j = 0; j < pnetinfo->itensor_num; j++) {
            bmrt_print_shape("max_itensor ",
                             j,
                             tensor_names[j],
                             pnetinfo->input_tensor_max_info[j].shape,
                             pnetinfo->input_tensor_max_info[j].dims);
            bmrt_print_shape("real_itensor",
                             j,
                             tensor_names[j],
                             &pnetinfo->input_shapes[shape_offset],
                             pnetinfo->input_dims[j]);
            shape_offset += pnetinfo->input_dims[j];
        }
        free(tensor_names);
        bmrt_get_output_tensor(bmrt_info.p_bmrt,
                               i,
                               NULL,
                               &tensor_num_,
                               &tensor_names);
        cout << "this net has " << pnetinfo->otensor_num
             << " output tensor" << endl;
        for (int j = 0; j < pnetinfo->otensor_num; j++) {
            bmrt_print_shape("max_otensor ",
                             j, tensor_names[j],
                             pnetinfo->output_tensor_max_info[j].shape,
                             pnetinfo->output_tensor_max_info[j].dims);
            bmrt_print_shape("real_otensor",
                             j, tensor_names[j],
                             pnetinfo->output_tensor_info[j].shape,
                             pnetinfo->output_tensor_info[j].dims);
        }
        cout << endl;
        pnetinfo++;
    }
}

void bmrt_delete_net_info(struct bmrt_info &bmrt_info) {
    for (int i = 0; i < bmrt_info.net_num; i++) {
        bmrt_net_info *pnetinfo = &bmrt_info.netinfo[i];
        delete[] pnetinfo->input_tensor_info;
        pnetinfo->input_tensor_info = NULL;
        delete[] pnetinfo->input_tensor_max_info;
        pnetinfo->input_tensor_max_info = NULL;
        delete[] pnetinfo->input_shapes;
        pnetinfo->input_shapes = NULL;
        delete[] pnetinfo->input_dims;
        pnetinfo->input_dims = NULL;
        delete[] pnetinfo->output_tensor_info;
        pnetinfo->output_tensor_info = NULL;
        delete[] pnetinfo->output_tensor_max_info;
        pnetinfo->output_tensor_max_info = NULL;
    }
    delete[] bmrt_info.netinfo;
    bmrt_info.netinfo = NULL;
}

void bmrt_alloc_memory(struct bmrt_info &bmrt_info) {
    for (int i = 0; i < bmrt_info.net_num; i++) {
        bmrt_net_info *pnetinfo = &bmrt_info.netinfo[i];
        /* alloc memory for input tensor*/
        pnetinfo->input_tensor_sys = new float *[pnetinfo->itensor_num];
        pnetinfo->input_mem_ping = new bm_device_mem_t[pnetinfo->itensor_num];
        int tensor_size = 0, shape_offset = 0;
        for (int j = 0; j < pnetinfo->itensor_num; j++) {
            tensor_size = bmrt_calcu_tensor_size(
                                    &pnetinfo->input_tensor_max_info[j]);
            pnetinfo->input_tensor_sys[j] = new float[tensor_size];
            bmrt_malloc_device_byte(bmrt_info.p_bmrt,
                                    &pnetinfo->input_mem_ping[j],
                                    tensor_size * sizeof(float));
        }
        /* alloc memory for output tensor*/
        pnetinfo->output_tensor_sys = new float *[pnetinfo->otensor_num];
        pnetinfo->output_mem_ping = new bm_device_mem_t[pnetinfo->otensor_num];
        for (int j = 0; j < pnetinfo->otensor_num; j++) {
            tensor_size = bmrt_calcu_tensor_size(
                                            &pnetinfo->output_tensor_info[j]);
            pnetinfo->output_tensor_sys[j] = new float[tensor_size];
            bmrt_malloc_device_byte(bmrt_info.p_bmrt,
                                    &pnetinfo->output_mem_ping[j],
                                    tensor_size * sizeof(float));
        }
    }
}

void bmrt_free_memory(struct bmrt_info &bmrt_info) {
    for (int i = 0; i < bmrt_info.net_num; i++) {
        bmrt_net_info *pnetinfo = &bmrt_info.netinfo[i];
        /* free output system and device memory */
        for (int j = 0; j < pnetinfo->otensor_num; j++) {
            delete[] pnetinfo->output_tensor_sys[j];
            pnetinfo->output_tensor_sys[j] = NULL;
            bmrt_free_device(bmrt_info.p_bmrt, pnetinfo->output_mem_ping[j]);
        }
        delete[] pnetinfo->output_tensor_info;
        pnetinfo->output_tensor_info = NULL;
        delete[] pnetinfo->output_tensor_max_info;
        pnetinfo->output_tensor_max_info = NULL;
        delete[] pnetinfo->output_tensor_sys;
        pnetinfo->output_tensor_sys = NULL;
        delete[] pnetinfo->output_mem_ping;
        pnetinfo->output_mem_ping = NULL;
        /* free input system and device memory */
        for (int j = 0; j < pnetinfo->itensor_num; j++) {
            delete[] pnetinfo->input_tensor_sys[j];
            pnetinfo->input_tensor_sys[j] = NULL;
            bmrt_free_device(bmrt_info.p_bmrt, pnetinfo->input_mem_ping[j]);
        }
        delete[] pnetinfo->input_shapes;
        pnetinfo->input_shapes = NULL;
        delete[] pnetinfo->input_dims;
        pnetinfo->input_dims = NULL;
        delete[] pnetinfo->input_tensor_info;
        pnetinfo->input_tensor_info = NULL;
        delete[] pnetinfo->input_tensor_max_info;
        pnetinfo->input_tensor_max_info = NULL;
        delete[] pnetinfo->input_tensor_sys;
        pnetinfo->input_tensor_sys = NULL;
        delete[] pnetinfo->input_mem_ping;
        pnetinfo->input_mem_ping = NULL;
    }
}

#endif  // SAMPLES_CPP_BMNNSDK_RUN_INCEPTIONV1_INCLUDE_UTIL_HPP_
