* Summary
```shell
SAIL C++ Samples List:
Classification:
      - mobilenetv1              (caffe)
      - mobilenetv1_tf           (tensorflow)
      - resnet50                 (caffe)
ObjectDetection:
      - yolov3                   (caffe)
      - mobilenetssd             (caffe)
FaceDetection:
      - mtcnn                    (caffe)
Others:
      - General Image Classification
      - Using VPU to decode video and BMCV to process image
      - Algorithm Performance Validation
```

Make sure the project root dir is the work dir.
```shell
cd sophon-inference
```

* Image classification for mobilenet_v1 of Caffe model.
```shell
# download model
python3 ./tools/downloader.py mobilenetv1
# complie to IR
python3 ./tools/convertbmodel.py mobilenetv1
# run the model
./build/bin/cls_mobilenetv1 \
      --context_dir=$HOME/.sophon/models/mobilenetv1_ir/ \
      --input_path=./tests/data/cls_imgs/cls_1.jpg
```

* Image classification for mobilenet_v1 of Tensorflow model.
```shell
# download model
python3 ./tools/downloader.py mobilenetv1_tf
# complie to IR
python3 ./tools/convertbmodel.py mobilenetv1_tf
# run the model
./build/bin/cls_mobilenetv1_tf \
      --context_dir=$HOME/.sophon/models/mobilenetv1_tf_ir/ \
      --input_path=./tests/data/cls_imgs/cls_1.jpg
```

* Image classification for resnet50 of Caffe model.
```shell
# download model
python3 ./tools/downloader.py resnet50
# complie to IR
python3 ./tools/convertbmodel.py resnet50
# run the model
./build/bin/cls_resnet50 \
      --context_dir=$HOME/.sophon/models/resnet50_ir/ \
      --input_path=./tests/data/cls_imgs/cls_1.jpg
```

* Object detection for mobilenetssd of Caffe model, with input of an image.
```shell
# download model
python3 ./tools/downloader.py mobilenetssd
# complie to IR
python3 ./tools/convertbmodel.py mobilenetssd
# run the model
./build/bin/det_mobilessd_image \
      --context_dir=$HOME/.sophon/models/mobilenetssd_ir/ \
      --input_path=./tests/data/det_1.jpg
```

* Object detection for mobilenetssd of Caffe model, with input of a video.
```shell
# download model
python3 ./tools/downloader.py mobilenetssd
# complie to IR
python3 ./tools/convertbmodel.py mobilenetssd
# run the model
./build/bin/det_mobilessd_video \
      --context_dir=$HOME/.sophon/models/mobilenetssd_ir/ \
      --input_path=./tests/data/car.mp4
```

* Object detection for mobilenetssd of Caffe model using VPU decoder.
```shell
# download model
python3 ./tools/downloader.py mobilenetssd
# complie to IR
python3 ./tools/convertbmodel.py mobilenetssd
# run the model
./build/bin/det_mobilessd_vpu \
      --context_dir=$HOME/.sophon/models/mobilenetssd_ir/ \
      --input_path=./tests/data/car.mp4
```

* Face detection for mtcnn of Caffe model.
```shell
# download model
python3 ./tools/downloader.py mtcnn
# complie to IR
python3 ./tools/convertbmodel.py mtcnncxx
# run the model
./build/bin/det_mtcnn \
      --context_dir=$HOME/.sophon/models/mtcnncxx_ir/ \
      --input_path=./tests/data/face_1.jpg
```

* Object detection for yolov3 of Caffe model.
```shell
# download model
python3 ./tools/downloader.py yolov3
# complie to IR
python3 ./tools/convertbmodel.py yolov3
# run the model
./build/bin/det_yolov3 \
      --context_dir=$HOME/.sophon/models/yolov3_ir/ \
      --input_path=./tests/data/det_1.jpg
```

* General image classification sample.
```shell
# download model
python3 ./tools/downloader.py resnet50
# complie to IR
python3 ./tools/convertbmodel.py resnet50
# run the model
./build/bin/img_cls \
      --context_dir=$HOME/.sophon/models/resnet50_ir/ \
      --input_dir=./tests/data/cls_imgs/
```

* Performance validation sample.
```shell
# download model
python3 ./tools/downloader.py resnet50
# complie to IR
python3 ./tools/convertbmodel.py resnet50
# run the model
./build/bin/performance_val \
      --context_dir=$HOME/.sophon/models/resnet50_ir \
      --batch=1 \
      --channel=3 \
      --height=224 \
      --width=224 \
      --sync_mode=sync
```
