* Summary
```shell
Python Samples List:
classification:
      - googlenet                (caffe)
      - resnet50                 (caffe)
      - mobilenetv1              (caffe)
      - vgg16                    (caffe)
      - mobilenetv1_tf           (tensorflow)
ObjectDetection:
      - yolov3                   (caffe)
      - mobilenetyolov3          (caffe)
      - mobilenetssd             (caffe)
      - fasterrcnn_vgg           (caffe)
      - fasterrcnn_resnet50_tf   (tensorflow)
FaceDetection:
      - mtcnn                    (caffe)
      - ssh                      (caffe)
SemanticSegmentation:
      - deeplabv3_mobilenetv2_tf (tensorflow)
```

Make sure the project root dir is the work dir.
```shell
cd sophon-inference
```

run yolov3 using sail
```shell
# download model
python3 ./tools/downloader.py yolov3
# complie to IR
python3 ./tools/convertbmodel.py yolov3
# run the model
python3 ./samples/python/run_yolov3.py \
    --ir_path $HOME/.sophon/models/yolov3_ir/ \
    --input_path ./tests/data/det_2.jpg
```

run models using algokit, a tool depends on sail

* Image classification for googlenet of Caffe model.
```shell
# download model
python3 ./tools/downloader.py googlenet
# complie to IR
python3 ./tools/convertbmodel.py googlenet
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name googlenet \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
```

* Image classification for resnet50 of Caffe model.
```shell
# download model
python3 ./tools/downloader.py resnet50
# complie to IR
python3 ./tools/convertbmodel.py resnet50
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name resnet50 \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
```

* Image classification for mobilenetv1 of Caffe model.
```shell
# download model
python3 ./tools/downloader.py mobilenetv1
# complie to IR
python3 ./tools/convertbmodel.py mobilenetv1
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name mobilenetv1 \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
```

* Image classification for vgg16 of Caffe model.
```shell
# download model
python3 ./tools/downloader.py vgg16
# complie to IR
python3 ./tools/convertbmodel.py vgg16
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name vgg16 \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
```

* Image classification for mobilenetv1 of Tensorflow model.
```shell
# download model
python3 ./tools/downloader.py mobilenetv1_tf
# complie to IR
python3 ./tools/convertbmodel.py mobilenetv1_tf
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name mobilenetv1_tf \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
```

* Object detection for yolov3 of Caffe model, with input of an image.
```shell
# download model
python3 ./tools/downloader.py yolov3
# complie to IR
python3 ./tools/convertbmodel.py yolov3
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name yolov3 \
    --input_path ./tests/data/det_2.jpg
```

* Object detection for mobilenetyolov3 yolov3 of Caffe model, with input of an image.
```shell
# download model
python3 ./tools/downloader.py mobilenetyolov3
# complie to IR
python3 ./tools/convertbmodel.py mobilenetyolov3
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name mobilenetyolov3 \
    --input_path ./tests/data/det_2.jpg
```

* Object detection for mobilenetssd of Caffe model, with input of an image.
```shell
# download model
python3 ./tools/downloader.py mobilenetssd
# complie to IR
python3 ./tools/convertbmodel.py mobilenetssd
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name mobilenetssd \
    --input_path ./tests/data/det_1.jpg
```

* Object detection for fasterrcnn_vgg of Caffe model, with input of an image.
```shell
# download model
python3 ./tools/downloader.py fasterrcnn_vgg
# complie to IR
python3 ./tools/convertbmodel.py fasterrcnn_vgg
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name fasterrcnn \
    --input_path ./tests/data/det_1.jpg
```

* Object detection for fasterrcnn_resnet50_tf of Tensorflow model, with input of an image.
```shell
# download model
python3 ./tools/downloader.py fasterrcnn_resnet50_tf
# complie to IR is included in next step
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name fasterrcnn_resnet50_tf \
    --input_path ./tests/data/det_1.jpg
```

* Face detection for mtcnn of Caffe model.
```shell
# download model
python3 ./tools/downloader.py mtcnn
# complie to IR
python3 ./tools/convertbmodel.py mtcnn
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type face_det \
    --algo_name mtcnn \
    --input_path ./tests/data/face_1.jpg
```

* Face detection for ssh of Caffe model.
```shell
# download model
python3 ./tools/downloader.py ssh
# complie to IR
python3 ./tools/convertbmodel.py ssh
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type face_det \
    --algo_name ssh \
    --input_path ./tests/data/face_1.jpg
```

* SemanticSegmentation for deeplabv3_mobilenetv2_tf of Tensorflow model.
```shell
# download model
python3 ./tools/downloader.py deeplabv3_mobilenetv2_tf
# complie to IR is included in next step
# run the model
python3 ./samples/python/run_algokit.py \
    --algo_type seg \
    --algo_name deeplabv3_mobilenetv2_tf \
    --input_path ./tests/data/seg_1.jpg
```
