/* Copyright 2016-2022 by Bitmain Technologies Inc. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

==============================================================================*/

#include "core/common.hpp"
#include "core/utils.hpp"
#include "core/bm_dnn.hpp"

namespace sail {

Bmdnn::Bmdnn(BmRuntime bmrt) : p_bmrt_(bmrt.get_ptr()),
    handle_(static_cast<bm_handle_t>(bmrt_get_bm_handle(bmrt.get_ptr()))) {}

int Bmdnn::sigmoid(Tensor<float>& input, Tensor<float>& output) {
  bm_device_mem_t* in = input.dev_data();
  bm_device_mem_t* out = output.dev_data();
  const vector<int>& in_shape = input.shape();
  const vector<int>& out_shape = output.shape();
  if (!shape_is_valid(in_shape) or !shape_is_valid(out_shape)) {
     cout << "Error: invalid tensor shape!" << endl;
     return -1;
  }
  if (!shapes_are_equal(in_shape, out_shape)) {
     cout << "Error: tensor shapes are not the same!" << endl;
     return -2;
  }
  int n = in_shape[0];
  int dim = get_size_of_shape(in_shape) / n;

  int ret = bmdnn_sigmoid_forward(handle_, *in, n, dim, *out);
  return ret;
}

}  // namespace sail
