/* Copyright 2016-2022 by Bitmain Technologies Inc. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.

==============================================================================*/

#include <iostream>
#include "core/common.hpp"
#include "core/statistics.hpp"

namespace sail {

vector<string> Statistics::get_items() {
  vector<string> items;
  for (map<string, vector<TimeSlice> >::const_iterator it = records_.begin();
      it != records_.end(); ++it) {
    items.push_back(it->first);
  }
  return items;
}

void Statistics::record_start(const string& item) {
  TimeSlice slice;
  clock_gettime(CLOCK_MONOTONIC, &(slice.start));
  records_[item].push_back(slice);
}

void Statistics::record_end(const string& item) {
  TimeSlice& slice = records_[item].back();
  clock_gettime(CLOCK_MONOTONIC, &(slice.end));
}

long int Statistics::timespec2ms(const struct timespec& ts) {
  return (long int) ts.tv_sec * 1000 + ts.tv_nsec / 1000 / 1000;
}

long int Statistics::get_time(const string& item) {
  if (records_.find(item) == records_.end()) {
    cout << "Wrong item name: " << item << endl;
    return 0;
  }
  long int time_ms = 0;
  for (vector<TimeSlice>::iterator it = records_[item].begin();
      it != records_[item].end(); ++it) {
    time_ms += timespec2ms(it->end) - timespec2ms(it->start);
  }
  return time_ms;
}

void Statistics::print_time() {
  for (map<string, vector<TimeSlice>>::iterator it = records_.begin();
      it != records_.end(); ++it) {
    cout << it->first << ": " << get_time(it->first) << " ms." << endl;
  }
}

}  // namespace sail
