# Copyright 2016-2022 Bitmain Technologies Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Run a image classification model of GoogleNet with input of images
"""

#from __future__ import print_function
import sys
import os
import unittest
import argparse
import cv2

def get_files_in_dir(dir_path):
  """ Get paths of the files in the specified directory.

  Args:
    dir_path: Directory path.

  Returns:
    list: A list of file paths.
  """
  files = []
  for file_ in os.listdir(dir_path):
#    file_path = os.path.join(dir_path, file_)
    files.append(file_)
  files.sort()
  return files


def cmp_result(result,true_result_m,output_shape,files_v) :
  offset = output_shape[1] * output_shape[2] * output_shape[3];
  batch = output_shape[0];
  index = 0
  for i in range(len(files_v)):
    files_name = files_v[i]
    if len(true_result_m[files_name]) != offset:
      return False
    for j in range(offset):
      true_result = true_result_m[files_name][j];
      if abs(true_result - result[j + index]) > 1.0e-6 :
       return False
    index = index + offset;
  return True

class TestImgCls(unittest.TestCase):
  """unit test class for image classification
  """
  def __init__(self,testname,param):
    self.sail_path = param
    self.CONTEXT_DIR= os.path.join(self.sail_path,"BM1682bmodel_c/resnet50_ir/")
    true_result_file = os.path.join(self.sail_path,"tests/data/resnet_static_result.txt")
    self.true_result_m = dict()
    mfile = open(true_result_file,"r")
    for eachline in mfile.readlines():
       line_buf = eachline.rstrip().split()
       file_name = line_buf[0]
       n = int(line_buf[1])
       c = int(line_buf[2])
       h = int(line_buf[3])
       w = int(line_buf[4])
       result = [float(i) for i in line_buf[5:]]
       self.true_result_m[file_name] = result

    super(TestImgCls, self).__init__(testname)

  def ResNet_static_sync_singal(self):
    inference = sail.Inference(self.CONTEXT_DIR, "BM1682_PCIE", "0", \
                                    sail.IOMode.SYSIO, False, False)
    preprocessor = sail.ImageNetPreprocess()
    graph_name = inference.get_all_graph_names()[0]
    input_names = inference.get_input_tensor_names(graph_name)
    input_shape = inference.get_input_shape(graph_name, input_names[0])
    output_names = inference.get_output_tensor_names(graph_name)
    output_shape = inference.get_output_shape(graph_name, output_names[0])
    input_tensor = inference.get_input_tensor_sys(graph_name, input_names[0])

    image_size = int(input_shape[1] * input_shape[2] * input_shape[3])
    batch_size = int(input_shape[0])
    class_num = int(output_shape[1])

    preprocessor = sail.ImageNetPreprocess()

    image_dir = os.path.join(self.sail_path, 'tests/data/pic_test/')
    image_files = get_files_in_dir(image_dir)
    input_num = len(image_files)
    offset = 0
    while (offset < input_num and input_num - offset >= batch_size):
      files_v = []
      for i in range(batch_size):
        image = cv2.imread(os.path.join(image_dir,image_files[offset + i]))
        preprocessor.process_cpu(input_tensor, i * image_size, image)
        files_v.append(image_files[offset+i])
        
       
      inference.process(graph_name)
      output_tensor = inference.get_output_tensor_sys(graph_name,
                                                      output_names[0])
      self.assertTrue(cmp_result(output_tensor.ravel(),self.true_result_m,output_shape,files_v))
      offset += batch_size


  def ResNet_static_sync_mutl(self):
    inference = sail.Inference(self.CONTEXT_DIR, "BM1682_PCIE", "all", \
                                    sail.IOMode.SYSIO, False, False)
    preprocessor = sail.ImageNetPreprocess()
    graph_name = inference.get_all_graph_names()[0]
    input_names = inference.get_input_tensor_names(graph_name)
    input_shape = inference.get_input_shape(graph_name, input_names[0])
    output_names = inference.get_output_tensor_names(graph_name)
    output_shape = inference.get_output_shape(graph_name, output_names[0])
    input_tensor = inference.get_input_tensor_sys(graph_name, input_names[0])

    image_size = int(input_shape[1] * input_shape[2] * input_shape[3])
    batch_size = int(input_shape[0])
    class_num = int(output_shape[1])

    preprocessor = sail.ImageNetPreprocess()

    image_dir = os.path.join(self.sail_path, 'tests/data/pic_test/')
    image_files = get_files_in_dir(image_dir)
    input_num = len(image_files)
    offset = 0
    while (offset < input_num and input_num - offset >= batch_size):
      files_v = []
      for i in range(batch_size):
        image = cv2.imread(os.path.join(image_dir,image_files[offset + i]))
        preprocessor.process_cpu(input_tensor, i * image_size, image)
        files_v.append(image_files[offset+i])
       
      # If data of input_tensor was prepared and it's type is numpy.array,
      # please copy data as following:
      #   inference.set_input_tensor_sys(graph_name, input_name, prepared_data)
      # or
      #   input_tensor = inference.get_input_tensor_sys(graph_name, input_name)
      #   np.copyto(input_tensor, prepared_data)
      inference.process(graph_name)
      output_tensor = inference.get_output_tensor_sys(graph_name,
                                                      output_names[0])
#      cmp_result(output_tensor.ravel(),self.true_result_m,output_shape,files_v)
      self.assertTrue(cmp_result(output_tensor.ravel(),self.true_result_m,output_shape,files_v))
#      print(output_tensor.data)
      offset += batch_size


  def ResNet_static_async_singal(self):
    inference = sail.Inference(self.CONTEXT_DIR, "BM1682_PCIE", "0", \
                                    sail.IOMode.SYSIO, False, True)
    preprocessor = sail.ImageNetPreprocess()
    graph_name = inference.get_all_graph_names()[0]
    input_names = inference.get_input_tensor_names(graph_name)
    input_shape = inference.get_input_shape(graph_name, input_names[0])
    output_names = inference.get_output_tensor_names(graph_name)
    output_shape = inference.get_output_shape(graph_name, output_names[0])
    input_tensor = inference.get_input_tensor_sys(graph_name, input_names[0])

    image_size = int(input_shape[1] * input_shape[2] * input_shape[3])
    batch_size = int(input_shape[0])
    class_num = int(output_shape[1])

    preprocessor = sail.ImageNetPreprocess()

    image_dir = os.path.join(self.sail_path, 'tests/data/pic_test/')
    image_files = get_files_in_dir(image_dir)
    input_num = len(image_files)
    offset = 0
    offset = 0
    mode = sail.TensorMode.PING
    status = sail.AsyncStatus.BEGIN
    files_v = []
    files_v_last = []
    while (offset < input_num and input_num - offset >= batch_size):
      files_v = []
      for i in range(batch_size):
        image = cv2.imread(os.path.join(image_dir,image_files[offset + i]))
        preprocessor.process_cpu(input_tensor, i * image_size, image)
        files_v.append(image_files[offset+i])

      # If data of input_tensor was prepared and it's type is numpy.array,
      # please copy the data in the way:
      #   np.copyto(input_tensor, prepared_data)
      inference.process(graph_name, mode, status)
      if status != sail.AsyncStatus.BEGIN:
        output_tensor = inference.get_output_tensor_sys( \
            graph_name, output_names[0])
        self.assertTrue(cmp_result(output_tensor.ravel(),self.true_result_m,output_shape,files_v_last))
        #class_ids = postprocessor.process(output_tensor)
        #print_result(output_tensor, class_ids)
      offset += batch_size
      mode = sail.TensorMode.PONG if (mode == sail.TensorMode.PING) \
             else sail.TensorMode.PING
      status = sail.AsyncStatus.PROCESS
      files_v_last =  files_v

    inference.process(graph_name, mode, sail.AsyncStatus.END)
    output_tensor = inference.get_output_tensor_sys(graph_name, output_names[0])
    self.assertTrue(cmp_result(output_tensor.ravel(),self.true_result_m,output_shape,files_v_last))
    #class_ids = postprocessor.process(output_tensor)
    #print_result(output_tensor, class_ids)


if __name__ == '__main__':
  if len(sys.argv) != 2:
    print("Usage: python pytest_img_cls.py sail_root")
    sys.exit(-1)

  root_path = sys.argv[1]
  sys.path.append(os.path.join(root_path, "./build/lib/"))
  import sail
  suite = unittest.TestSuite()
  suite.addTest(TestImgCls("ResNet_static_sync_singal",root_path ))
  suite.addTest(TestImgCls("ResNet_static_sync_mutl",root_path ))
  suite.addTest(TestImgCls("ResNet_static_async_singal",root_path ))

  unittest.TextTestRunner().run(suite)
