"""sophon sample models downloader
Usage: python downloader.py googlenet
"""
from __future__ import print_function
import sys, os
import six.moves.urllib as urllib
import tarfile, zipfile
import argparse

sophon_model_dict = {
    'fasterrcnn_resnet50_tf': 'fasterrcnn_resnet50_tf.tar.gz',
    'deeplabv3_mobilenetv2_tf': 'deeplabv3_mobilenetv2_tf.tar.gz',
    'fasterrcnn_vgg': 'fasterrcnn_vgg.tar.gz',
    'mtcnn': 'mtcnn.tar.gz',
    'ssh': 'ssh.tar.gz',
    'yolov3': 'yolov3.tar.gz',
    'googlenet': 'googlenet.tar.gz',
    'resnet50': 'resnet50.tar.gz',
    'vgg16': 'vgg16.tar.gz',
    'mobilenetv1': 'mobilenetv1.tar.gz',
    'mobilenetv1_tf': 'mobilenetv1_tf.tar.gz',
    'mobilenetssd': 'mobilenetssd.tar.gz',
    'mobilenetyolov3': 'mobilenetyolov3.tar.gz',
    'ffmpeg_squeezenet': 'ffmpeg_squeezenet.tar.gz'
}

def models_download(args, download_base_path, save_base_path):
    if args.model_name in sophon_model_dict.keys():
        model_file = sophon_model_dict[args.model_name]
    else:
        raise ValueError('not a available model_name', args.model_name)
    model_path = os.path.join(save_base_path, args.model_name)
    if os.path.exists(model_path):
        print("{} already exists!".format(args.model_name))
        sys.exit()
    print("{} download...".format(model_file))
    url_path = os.path.join(download_base_path, model_file)
    opener = urllib.request.URLopener()
    opener.retrieve(url_path, model_file)
    print("{} download finished!".format(model_file))
    print("unzip {} to {}".format(model_file, model_path))
    with tarfile.open(model_file) as tar_file:
        tar_file.extractall(save_base_path)
    os.remove(model_file)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='download sophon sample model')
    parser.add_argument("model_name", help="download model_name")
    args = parser.parse_args()
    download_base_path = 'https://sophon-file.bitmain.com.cn/sophon-prod/model/19/05/28/'
    save_base_path = os.path.join(os.path.expandvars('$HOME'), '.sophon/models')
    if os.path.exists(save_base_path):
        print("sophon model path already exists!")
    else:
        os.makedirs(save_base_path)
    models_download(args, download_base_path, save_base_path)
