# Install dependencies
# cd $PROJECT_ROOT/
/bin/bash ./tools/install.sh

# Download all needed models and compile them for tests.
MODELS="
       fasterrcnn_resnet50_tf \
       deeplabv3_mobilenetv2_tf \
       fasterrcnn_vgg \
       mtcnn \
       ssh \
       yolov3 \
       googlenet \
       resnet50 \
       vgg16 \
       mobilenetv1 \
       mobilenetv1_tf \
       mobilenetssd \
       mobilenetyolov3 \
       ffmpeg_squeezenet
      "
# Please make sure bmnett is installed for python3
for MODEL in ${MODELS[@]}
do
    python3 ./tools/downloader.py $MODEL
    if [[ $MODEL != "fasterrcnn_resnet50_tf" ]] && [[ $MODEL != "deeplabv3_mobilenetv2_tf" ]]; then
        python3 ./tools/convertbmodel.py $MODEL
    fi
    if [[ $MODEL == "mtcnn" ]]; then
        python3 ./tools/convertbmodel.py ${MODEL}cxx
    fi
done
exit 0

# Test cpp samples
# 1) set BMNNSDK installation path in $PROJECT_ROOT/CMakeLists.txt
# 2) set option CXX_API ON and set option PYTHON_API OFF in $PROJECT_ROOT/CMakeLists.txt
# 3) cd $PROJECT_ROOT and compile as following
mkdir build
cd build
cmake ..
cd ..

# Test cpp samples using BMNSDK
./build/bin/run_inceptionv1 $HOME/.sophon/models/googlenet_ir ./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "run_inceptionv1 failed!"
  exit -1
fi

./build/bin/vpu_decode -v ./tests/data/fenix5plus.mp4 -t 1 -i 0
if [[ $? != 0 ]]; then
  echo "vpu_decode failed!"
  exit -1
fi

./build/bin/test_ffmpeg_squeezenet \
    -d $HOME/.sophon/models/ffmpeg_squeezenet_ir -h 0.8 \
    -v ./tests/data/fenix5plus.mp4 \
       ./tests/data/fenix5plus.mp4 \
       ./tests/data/fenix5plus.mp4 \
       ./tests/data/fenix5plus.mp4 \
    -p ./tests/data/ref_new.jpg -t 4 -i 0
if [[ $? != 0 ]]; then
  echo "test_ffmpeg_squeezenet failed!"
  exit -1
fi

echo "Tests of cpp samples using BMNSDK Passed!"
echo ""
#exit 0

# Test cpp samples using SAIL
./build/bin/cls_mobilenetv1 \
      --context_dir=$HOME/.sophon/models/mobilenetv1_ir/ \
      --input_path=./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "cls_mobilenetv1 failed!"
  exit -1
fi

./build/bin/cls_mobilenetv1_tf \
      --context_dir=$HOME/.sophon/models/mobilenetv1_tf_ir/ \
      --input_path=./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "cls_mobilenetv1_tf failed!"
  exit -1
fi

./build/bin/cls_resnet50 \
      --context_dir=$HOME/.sophon/models/resnet50_ir/ \
      --input_path=./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "cls_resnet50 failed!"
  exit -1
fi

./build/bin/det_mobilessd_image \
      --context_dir=$HOME/.sophon/models/mobilenetssd_ir/ \
      --input_path=./tests/data/det_1.jpg
if [[ $? != 0 ]]; then
  echo "det_mobilessd_image failed!"
  exit -1
fi

./build/bin/det_mobilessd_video \
      --context_dir=$HOME/.sophon/models/mobilenetssd_ir/ \
      --input_path=./tests/data/car.mp4
if [[ $? != 0 ]]; then
  echo "det_mobilessd_video failed!"
  exit -1
fi

./build/bin/det_mobilessd_vpu \
      --context_dir=$HOME/.sophon/models/mobilenetssd_ir/ \
      --input_path=./tests/data/car.mp4
if [[ $? != 0 ]]; then
  echo "det_mobilessd_vpu failed!"
  exit -1
fi

./build/bin/det_mtcnn \
      --context_dir=$HOME/.sophon/models/mtcnncxx_ir/ \
      --input_path=./tests/data/face_1.jpg
if [[ $? != 0 ]]; then
  echo "det_mtcnn failed!"
  exit -1
fi

./build/bin/det_yolov3 \
      --context_dir=$HOME/.sophon/models/yolov3_ir/ \
      --input_path=./tests/data/det_1.jpg
if [[ $? != 0 ]]; then
  echo "det_yolov3 failed!"
  exit -1
fi

./build/bin/img_cls \
      --context_dir=$HOME/.sophon/models/resnet50_ir/ \
      --input_dir=./tests/data/cls_imgs/
if [[ $? != 0 ]]; then
  echo "img_cls failed!"
  exit -1
fi

./build/bin/performance_val \
      --context_dir=$HOME/.sophon/models/resnet50_ir \
      --batch=1 \
      --channel=3 \
      --height=224 \
      --width=224 \
      --sync_mode=sync
if [[ $? != 0 ]]; then
  echo "performance_val failed!"
  exit -1
fi
echo "Tests of cpp samples using SAIL Passed!"
echo ""

# Test python samples using AlgoKit
# 1) set BMNNSDK installation path in $PROJECT_ROOT/CMakeLists.txt
# 2) set option CXX_API OFF and set option PYTHON_API ON in $PROJECT_ROOT/CMakeLists.txt
# 3) set boost_python paths in $PROJECT_ROOT/CMakeLists.txt
# 4) cd $PROJECT_ROOT and compile as following
cd build
rm -rf ./*
cmake ..
cd ..
cd ./python
source ./setup_python_path.sh
cd ..

python3 ./samples/python/run_yolov3.py \
    --ir_path $HOME/.sophon/models/yolov3_ir/ \
    --input_path ./tests/data/det_2.jpg
if [[ $? != 0 ]]; then
  echo "run_yolov3.py failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name googlenet \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py googlenet failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name resnet50 \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py resnet50 failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name mobilenetv1 \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py mobilenetv1 failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name vgg16 \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py vgg16 failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type cls \
    --algo_name mobilenetv1_tf \
    --input_path ./tests/data/cls_imgs/cls_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py mobilenetv1_tf failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name yolov3 \
    --input_path ./tests/data/det_2.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py yolov3 failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name mobilenetyolov3 \
    --input_path ./tests/data/det_2.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py mobilenetyolov3 failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name mobilenetssd \
    --input_path ./tests/data/det_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py mobilenetssd failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name fasterrcnn \
    --input_path ./tests/data/det_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py fasterrcnn failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type obj_det \
    --algo_name fasterrcnn_resnet50_tf \
    --input_path ./tests/data/det_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py fasterrcnn_resnet50_tf failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type face_det \
    --algo_name mtcnn \
    --input_path ./tests/data/face_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py mtcnn failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type face_det \
    --algo_name ssh \
    --input_path ./tests/data/face_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py ssh failed!"
  exit -1
fi

python3 ./samples/python/run_algokit.py \
    --algo_type seg \
    --algo_name deeplabv3_mobilenetv2_tf \
    --input_path ./tests/data/seg_1.jpg
if [[ $? != 0 ]]; then
  echo "run_algokit.py deeplabv3_mobilenetv2_tf failed!"
  exit -1
fi

echo "Tests ofpython samples using AlgoKit passed!"
echo ""

echo "All tests passed!"
